# -*- coding: utf-8 -*-
"""

ModelViewDelegate for an MPL Line object

Created on Thu Jul  7 10:01:23 2016

@author: chc
"""

from PyQt5.QtWidgets import (QApplication as _QApplication,
                             QMainWindow as _QMainWindow,
                             QColorDialog as _QColorDialog,
                             QDoubleSpinBox as _QDoubleSpinBox,
                             QComboBox as _QComboBox,
                             QLineEdit as _QLineEdit,
                             QStyledItemDelegate as _QStyledItemDelegate,
                             QTableView as _QTableView,
                             QSizePolicy as _QSizePolicy)

from PyQt5.QtCore import (QAbstractTableModel as _QAbstractTableModel,
                          QVariant as _QVariant,
                          QObject as _QObject,
                          pyqtSignal as _pyqtSignal,
                          QModelIndex as _QModelIndex,
                          Qt as _Qt)

from PyQt5.QtGui import (QPixmap as _QPixmap,
                         QIcon as _QIcon,
                         QColor as _QColor)


from sciplot.ui.models.abstract import (AbstractTableModelMpl as
                                        _AbstractTableModelMpl,
                                        AbstractEditDelegateMpl as
                                        _AbstractEditDelegateMpl)
from sciplot.utils.general import round_list

class TableModelBars(_AbstractTableModelMpl):
    _HEADERS = ['Facecolor',
                'Alpha',
                'Edgecolor',
                'Line Width',
                'Width Factor',
                'Label',
                'Delete']

    _COL_FACECOLOR = _HEADERS.index('Facecolor')
    _COL_ALPHA = _HEADERS.index('Alpha')
    _COL_EDGECOLOR = _HEADERS.index('Edgecolor')
    _COL_LINEWIDTH = _HEADERS.index('Line Width')
    _COL_WIDTH_FACTOR = _HEADERS.index('Width Factor')
    _COL_LABEL = _HEADERS.index('Label')
    _COL_DELETE = _HEADERS.index('Delete')

    # Emit row, plot ID
    dataDeleted = _pyqtSignal(int, float)

    def __init__(self, parent=None):

        super(_QAbstractTableModel, self).__init__(parent)
        self.headers = TableModelBars._HEADERS
        self._model_data = []

    def rowCount(self, parent=_QModelIndex()):
            return len(self._model_data)

    def columnCount(self, parent=_QModelIndex()):
        return len(self.headers)

    def headerData(self, col, orientation, role):
        if orientation == _Qt.Horizontal and role == _Qt.DisplayRole:
            return self.headers[col]
        return _QVariant()

    def doubleClickCheck(self, index):
        col = index.column()
        if (col == TableModelBars._COL_FACECOLOR or
                col == TableModelBars._COL_EDGECOLOR):  # Colors
            self.changeColor(index)
        elif col == TableModelBars._COL_DELETE:  # Delete?
            self.deleteData(index)

    def deleteData(self, index):
            self.setData(index, True)

    def changeColor(self, index):
        row = index.row()
        col = index.column()

        if col == TableModelBars._COL_FACECOLOR:
            color = self._model_data[row]['facecolor']
        elif col == TableModelBars._COL_EDGECOLOR:
            color = self._model_data[row]['edgecolor']
        color_256 = [color[0]*255, color[1]*255, color[2]*255]
        qcolor = _QColor(color_256[0], color_256[1], color_256[2])

        result = _QColorDialog.getColor(qcolor)
        if _QColor.isValid(result):
            self.setData(index, result.getRgb())
        else:
            return None

    def data(self, index, role=_Qt.DisplayRole):
        if not index.isValid() or not 0 <= index.row() < self.rowCount():
            return _QVariant()

        row = index.row()
        col = index.column()

        if role == _Qt.DisplayRole:
            if col == TableModelBars._COL_FACECOLOR:
                return str(round_list(self._model_data[row]['facecolor']))
            elif col == TableModelBars._COL_ALPHA:
                return str(self._model_data[row]['alpha'])
            elif col == TableModelBars._COL_EDGECOLOR:
                return str(round_list(self._model_data[row]['edgecolor']))
            elif col == TableModelBars._COL_LINEWIDTH:
                return str(self._model_data[row]['linewidth'])
            elif col == TableModelBars._COL_WIDTH_FACTOR:
                return str(self._model_data[row]['width_factor'])
            elif col == TableModelBars._COL_LABEL:
                return str(self._model_data[row]['label'])
            elif col == TableModelBars._COL_DELETE:
                return ''
        elif role == _Qt.DecorationRole:
            if col == TableModelBars._COL_FACECOLOR:
                color = self._model_data[row]['facecolor']
            elif col == TableModelBars._COL_EDGECOLOR:
                color = self._model_data[row]['edgecolor']
            if (col == TableModelBars._COL_FACECOLOR or
                    col == TableModelBars._COL_EDGECOLOR):
                color_256 = [color[0]*255, color[1]*255, color[2]*255]
                qcolor = _QColor(color_256[0], color_256[1], color_256[2])
                pm = _QPixmap(20, 20)
                pm.fill(qcolor)
                icon = _QIcon(pm)
                return icon
            elif col == TableModelBars._COL_DELETE:
                color = [1, 0, 0]
                color_256 = [color[0]*255, color[1]*255, color[2]*255]
                qcolor = _QColor(color_256[0], color_256[1], color_256[2])
                pm = _QPixmap(20, 20)
                pm.fill(qcolor)
                icon = _QIcon(pm)
                return icon
            else:
                pass
        else:
            return _QVariant()

    def setData(self, index, value, role=_Qt.EditRole):
        if role == _Qt.EditRole:
            row = index.row()
            col = index.column()

            if (col == TableModelBars._COL_FACECOLOR or
                    col == TableModelBars._COL_EDGECOLOR):
                color_255 = value[0:-1]
                color = [round(color_255[0]/255, 2),
                         round(color_255[1]/255, 2),
                         round(color_255[2]/255, 2)]
                if col == TableModelBars._COL_FACECOLOR:
                    self._model_data[row]['facecolor'] = color
                if col == TableModelBars._COL_EDGECOLOR:
                    self._model_data[row]['edgecolor'] = color
            elif col == TableModelBars._COL_ALPHA:
                self._model_data[row]['alpha'] = float(value)
            elif col == TableModelBars._COL_LINEWIDTH:
                self._model_data[row]['linewidth'] = float(value)
            elif col == TableModelBars._COL_WIDTH_FACTOR:
                self._model_data[row]['width_factor'] = float(value)
            elif col == TableModelBars._COL_LABEL:
                self._model_data[row]['label'] = value
            elif col == TableModelBars._COL_DELETE:
                if value:
                    out = self._model_data.pop(row)
                    self.layoutChanged.emit()
                    self.dataDeleted.emit(row, out['id'])

            self.dataChanged.emit(index, index)

    def flags(self, index):
            flag = super(_QAbstractTableModel, self).flags(index)
            return flag | _Qt.ItemIsEditable


class EditDelegateBars(_AbstractEditDelegateMpl):
    def createEditor(self, parent, option, index):
            col = index.column()
            if col == TableModelBars._COL_FACECOLOR:
                # Color handled by doubleClicked SIGNAL
                pass
            elif col == TableModelBars._COL_EDGECOLOR:
                # Color handled by doubleClicked SIGNAL
                pass
            # Alpha
            elif col == TableModelBars._COL_ALPHA:
                spinBoxSize = _QDoubleSpinBox(parent)
                spinBoxSize.setMinimum(0)
                spinBoxSize.setMaximum(1)
                spinBoxSize.setSingleStep(.1)
                return spinBoxSize
            # Width Fraction
            elif col == TableModelBars._COL_WIDTH_FACTOR:
                spinBoxSize = _QDoubleSpinBox(parent)
                spinBoxSize.setMinimum(0)
                spinBoxSize.setMaximum(1000000000)
                spinBoxSize.setSingleStep(.1)
                return spinBoxSize

            # LineWidth
            elif col == TableModelBars._COL_LINEWIDTH:
                spinBoxSize = _QDoubleSpinBox(parent)
                spinBoxSize.setMinimum(0)
                spinBoxSize.setMaximum(20)
                spinBoxSize.setSingleStep(.5)
                return spinBoxSize
            elif col == TableModelBars._COL_LABEL:  # Label
                lineEditLabel = _QLineEdit(parent)
                return lineEditLabel
            elif col == TableModelBars._COL_DELETE:  # Delete?
                pass
            else:
                return _QVariant()

    def setEditorData(self, editor, index):
        col = index.column()
        item = index.data(_Qt.DisplayRole)
        if (col == TableModelBars._COL_FACECOLOR or
                col == TableModelBars._COL_EDGECOLOR):
            # Color handled by doubleClick SIGNAL
            pass
        # LineWidth or MarkerSize or Alpha
        elif (col == TableModelBars._COL_LINEWIDTH or
              col == TableModelBars._COL_ALPHA or
              col == TableModelBars._COL_WIDTH_FACTOR):
            item_float = float(item)
            editor.setValue(item_float)
        elif col == TableModelBars._COL_LABEL:  # Label
            editor.setText(item)
        else:
            pass

    def setModelData(self, editor, model, index):
        col = index.column()
        if col == TableModelBars._COL_FACECOLOR:
            # Color handled by doubleClick SIGNAL
            pass
        elif col == TableModelBars._COL_EDGECOLOR:
            # Color handled by doubleClick SIGNAL
            pass
        # LineWidth or MarkerSize or Alpha
        elif (col == TableModelBars._COL_LINEWIDTH or
              col == TableModelBars._COL_ALPHA or
              col == TableModelBars._COL_WIDTH_FACTOR):
            value = editor.value()
            model.setData(index, value)
        elif col == TableModelBars._COL_LABEL:  # Label
            label = editor.text()
            model.setData(index, label)



from PyQt5.QtWidgets import (QWidget, QDialogButtonBox, QApplication, 
                             QDialog, QHBoxLayout, QVBoxLayout,
                             QLabel, QDoubleSpinBox, QSpinBox)
import sys

class DualEntry(QDialog):
    """ Just a simple Dialog for retrieving 2 values """    
    def __init__(self, entry1, entry2, input_type, text, min, max, parent=None):
        """
        Note: Use the static method
        """
        super().__init__(parent=parent)
        self.initUI(entry1, entry2, input_type, text, min, max)
        
    def initUI(self, entry1, entry2, input_type, text, min, max):      
        self.setObjectName('Dialog')
        self.setStyleSheet("font: 10pt \"Arial\";")
        self.setWindowTitle('Input dialog')
        
        # Setting the parent makes it the default layout
        self.vlayout = QVBoxLayout(self)
        self.vlayout.setObjectName('vlayout')
        
        self.main_text = QLabel(self)
        self.main_text.setText(text)
        self.vlayout.insertWidget(0, self.main_text)
        self.hlayout = QHBoxLayout()
        self.hlayout.setObjectName('hlayout')
        self.vlayout.insertLayout(1,self.hlayout)

        if input_type is float:
            self.entry1 = QDoubleSpinBox(self)
            self.entry2 = QDoubleSpinBox(self)
        elif input_type is int:
            self.entry1 = QSpinBox(self)
            self.entry2 = QSpinBox(self)
        self.entry1.setMaximum(max)
        self.entry1.setMinimum(min)
        self.entry2.setMaximum(max)
        self.entry2.setMinimum(min)
        self.entry1.setValue(entry1)
        self.entry2.setValue(entry2)

        self.hlayout.insertWidget(0, self.entry1)
        self.hlayout.insertWidget(1, self.entry2)

        self.bb = QDialogButtonBox(self)
        self.bb.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)
        self.bb.accepted.connect(self.accept)
        self.bb.rejected.connect(self.reject)
        self.vlayout.insertWidget(-1,self.bb)
        self.show()

    @staticmethod
    def getDualEntries(entry1, entry2, input_type=float, text='Enter Values:', min=-10000000000,
                       max=10000000000, parent=None):
        """
        Pops up a dialog box that retrieves 2 values, either float or int

        Parameters
        ----------
        entry1: int or float
            Default value in the left box

        entry2: int or float
            Default value in the right box

        input_type : type
            Either float or int are currently supported

        text : str
            Text describing the values to set

        min : int or float
            Minimum value for entries

        max : int or float
            Maximum value for entries

        parent : object or None
            Parent of the Dialog (QDialog) box.

        Returns
        -------
        (numeric, numeric), int
            Returns (left value, right value), Ok pressed. If Ok was pressed, value == 1, else 0.    

        """
        dialog = DualEntry(entry1, entry2, input_type, text, min, max, parent)
        result = dialog.exec_()
        if result:
            return (dialog.entry1.value(), dialog.entry2.value()), result
        else:
            return (), result

if __name__ == '__main__':
    
    app = QApplication(sys.argv)
    ex = DualEntry.getDualEntries(0, 100, float, 'Tets', 0, 100)
    print(ex)
    # sys.exit(app.exec_())


# -*- coding: utf-8 -*-
"""
Created on Fri Jul  8 14:11:40 2016

@author: chc
"""

import matplotlib as _mpl

from PyQt5.QtWidgets import (QApplication as _QApplication,
                             QMainWindow as _QMainWindow,
                             QColorDialog as _QColorDialog,
                             QCheckBox as _QCheckBox,
                             QDoubleSpinBox as _QDoubleSpinBox,
                             QComboBox as _QComboBox,
                             QLineEdit as _QLineEdit,
                             QStyledItemDelegate as _QStyledItemDelegate,
                             QTableView as _QTableView,
                             QSizePolicy as _QSizePolicy)

from PyQt5.QtCore import (QAbstractTableModel as _QAbstractTableModel,
                          QVariant as _QVariant,
                          QObject as _QObject,
                          pyqtSignal as _pyqtSignal,
                          QModelIndex as _QModelIndex,
                          Qt as _Qt)

from PyQt5.QtGui import (QPixmap as _QPixmap,
                         QIcon as _QIcon,
                         QColor as _QColor)

from sciplot.utils.mplstyle import MplMarkers, MplLines

from sciplot.ui.models.abstract import (AbstractTableModelMpl as
                                        _AbstractTableModelMpl,
                                        AbstractEditDelegateMpl as
                                        _AbstractEditDelegateMpl)

class TableModelImages(_AbstractTableModelMpl):
    _HEADERS = ['Cmap',
                'Alpha',
                'Clim Low',
                'Clim High',
                'Label',
                'Colorbar',
                'Delete']

    _COL_CMAP = _HEADERS.index('Cmap')
    _COL_ALPHA = _HEADERS.index('Alpha')
    _COL_CLIM_LOW = _HEADERS.index('Clim Low')
    _COL_CLIM_HIGH = _HEADERS.index('Clim High')
    _COL_CBAR = _HEADERS.index('Colorbar')
    _COL_LABEL = _HEADERS.index('Label')
    _COL_DELETE = _HEADERS.index('Delete')

    dataDeleted = _pyqtSignal(int, float)

    def __init__(self, parent=None):

        super(_QAbstractTableModel, self).__init__(parent)
        self.headers = TableModelImages._HEADERS
        self._model_data = []

    def rowCount(self, parent=_QModelIndex()):
            return len(self._model_data)

    def columnCount(self, parent=_QModelIndex()):
        return len(self.headers)

    def headerData(self, col, orientation, role):
        if orientation == _Qt.Horizontal and role == _Qt.DisplayRole:
            return self.headers[col]
        return _QVariant()

    def doubleClickCheck(self, index):
        col = index.column()
#        if col == TableModelImages._COL_CMAP:  # CMAP
#            self.changeColor(index)
        if col == TableModelImages._COL_DELETE:  # Delete?
            self.deleteData(index)

    def deleteData(self, index):
            self.setData(index, True)

    def data(self, index, role=_Qt.DisplayRole):
        if not index.isValid() or not 0 <= index.row() < self.rowCount():
            return _QVariant()

        row = index.row()
        col = index.column()

        if role == _Qt.DisplayRole:
            if col == TableModelImages._COL_CMAP:
                return str(self._model_data[row]['cmap_name'])
            elif col == TableModelImages._COL_ALPHA:
                return str(self._model_data[row]['alpha'])
            elif col == TableModelImages._COL_CLIM_LOW:
                return str(self._model_data[row]['clim_low'])
            elif col == TableModelImages._COL_CLIM_HIGH:
                return str(self._model_data[row]['clim_high'])
            elif col == TableModelImages._COL_CBAR:
#                print('1')
                return str(self._model_data[row]['colorbar'])
            elif col == TableModelImages._COL_LABEL:
                return str(self._model_data[row]['label'])
            elif col == TableModelImages._COL_DELETE:
                return ''
        elif role == _Qt.DecorationRole:
            if col == TableModelImages._COL_DELETE:
                color = [1, 0, 0]
                color_256 = [color[0]*255, color[1]*255, color[2]*255]
                qcolor = _QColor(color_256[0], color_256[1], color_256[2])
                pm = _QPixmap(20, 20)
                pm.fill(qcolor)
                icon = _QIcon(pm)
                return icon
        else:
            return _QVariant()

    def setData(self, index, value, role=_Qt.EditRole):
        if role == _Qt.EditRole:
            row = index.row()
            col = index.column()

            if col == TableModelImages._COL_CMAP:
                self._model_data[row]['cmap_name'] = value
            elif col == TableModelImages._COL_ALPHA:
                self._model_data[row]['alpha'] = float(value)
            elif col == TableModelImages._COL_CLIM_LOW:
                self._model_data[row]['clim_low'] = float(value)
            elif col == TableModelImages._COL_CLIM_HIGH:
                self._model_data[row]['clim_high'] = float(value)
            elif col == TableModelImages._COL_CBAR:
#                print('2')
                self._model_data[row]['colorbar'] = bool(value)
            elif col == TableModelImages._COL_LABEL:
                self._model_data[row]['label'] = value
            elif col == TableModelImages._COL_DELETE:
                if value:
                    out = self._model_data.pop(row)
                    self.layoutChanged.emit()
                    self.dataDeleted.emit(row, out['id'])

            self.dataChanged.emit(index, index)

    def flags(self, index):
            flag = super(_QAbstractTableModel, self).flags(index)
            return flag | _Qt.ItemIsEditable


class EditDelegateImages(_AbstractEditDelegateMpl):
    def createEditor(self, parent, option, index):
            col = index.column()

            if col == TableModelImages._COL_ALPHA:
                spinBoxSize = _QDoubleSpinBox(parent)
                spinBoxSize.setMinimum(0)
                spinBoxSize.setMaximum(1)
                spinBoxSize.setSingleStep(.1)
                return spinBoxSize
            # clim_low, clim_high
            elif (col == TableModelImages._COL_CLIM_LOW or
                  col == TableModelImages._COL_CLIM_HIGH):
                spinBoxSize = _QDoubleSpinBox(parent)
                spinBoxSize.setMinimum(-1e10)
                spinBoxSize.setMaximum(1e10)
                spinBoxSize.setSingleStep(.5)
                return spinBoxSize
            elif col == TableModelImages._COL_CBAR:  # colorbar
#                print('3')
                comboBoxTrueFalse = _QComboBox(parent)
                comboBoxTrueFalse.addItems(['True','False'])
                return comboBoxTrueFalse
            elif col == TableModelImages._COL_CMAP:  # cmaps
                comboBoxCmapNames = _QComboBox(parent)
                list_cmaps = list(_mpl.cm.cmap_d.keys())
                list_cmaps.sort()
                for cmap_name in list_cmaps:
                    comboBoxCmapNames.addItem(cmap_name)
                return comboBoxCmapNames
            elif col == TableModelImages._COL_LABEL:  # Label
                lineEditLabel = _QLineEdit(parent)
                return lineEditLabel
            elif col == TableModelImages._COL_DELETE:  # Delete?
                pass
            else:
                return _QVariant()

    def setEditorData(self, editor, index):
        col = index.column()
        item = index.data(_Qt.DisplayRole)
        if (col == TableModelImages._COL_ALPHA or
              col == TableModelImages._COL_CLIM_LOW or
              col == TableModelImages._COL_CLIM_HIGH):
            item_float = float(item)
            editor.setValue(item_float)
        elif col == TableModelImages._COL_LABEL:  # Label
            editor.setText(item)
        elif (col == TableModelImages._COL_CBAR or
              col == TableModelImages._COL_CMAP): # colorbar or cmap
            loc = editor.findText(item)
            if loc < 0:
                loc = 0
            editor.setCurrentIndex(loc)
        else:
            pass

    def setModelData(self, editor, model, index):
        col = index.column()

        # Alpha or clim's
        if (col == TableModelImages._COL_ALPHA or
                col == TableModelImages._COL_CLIM_LOW or
                col == TableModelImages._COL_CLIM_HIGH):
            value = editor.value()
            model.setData(index, value)
        elif col == TableModelImages._COL_CMAP:  # cmap
            list_cmaps = list(_mpl.cm.cmap_d.keys())
            list_cmaps.sort()
            idx = editor.currentIndex()
            cmap_name = list_cmaps[idx]
            model.setData(index, cmap_name)
        elif col == TableModelImages._COL_CBAR:  # Colorbar
#            print('5')
            cbar = editor.currentText() == 'True'
            model.setData(index, cbar)
        elif col == TableModelImages._COL_LABEL:  # Label
            label = editor.text()
            model.setData(index, label)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '.\ui_Plotter.ui'
#
# Created by: PyQt5 UI code generator 5.12
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1208, 911)
        MainWindow.setStyleSheet("background-color: rgb(255, 255, 255);")
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setAutoFillBackground(False)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setSizeConstraint(QtWidgets.QLayout.SetNoConstraint)
        self.verticalLayout.setObjectName("verticalLayout")
        spacerItem = QtWidgets.QSpacerItem(1000, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout.addItem(spacerItem)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem1)
        self.horizontalLayout_4.addLayout(self.verticalLayout)
        self.groupBox = QtWidgets.QGroupBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setMaximumSize(QtCore.QSize(600, 16777215))
        self.groupBox.setBaseSize(QtCore.QSize(300, 0))
        self.groupBox.setTitle("")
        self.groupBox.setFlat(False)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_5.setSizeConstraint(QtWidgets.QLayout.SetNoConstraint)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.toolBox = QtWidgets.QToolBox(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.toolBox.sizePolicy().hasHeightForWidth())
        self.toolBox.setSizePolicy(sizePolicy)
        self.toolBox.setMinimumSize(QtCore.QSize(200, 0))
        self.toolBox.setMaximumSize(QtCore.QSize(300, 16777215))
        self.toolBox.setBaseSize(QtCore.QSize(100, 0))
        self.toolBox.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.toolBox.setObjectName("toolBox")
        self.FigureParameters = QtWidgets.QWidget()
        self.FigureParameters.setGeometry(QtCore.QRect(0, 0, 200, 610))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.FigureParameters.sizePolicy().hasHeightForWidth())
        self.FigureParameters.setSizePolicy(sizePolicy)
        self.FigureParameters.setObjectName("FigureParameters")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.FigureParameters)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_10 = QtWidgets.QLabel(self.FigureParameters)
        self.label_10.setObjectName("label_10")
        self.gridLayout_2.addWidget(self.label_10, 2, 0, 1, 1)
        self.spinBoxFigureDPI = QtWidgets.QSpinBox(self.FigureParameters)
        self.spinBoxFigureDPI.setMinimum(10)
        self.spinBoxFigureDPI.setMaximum(100000000)
        self.spinBoxFigureDPI.setSingleStep(100)
        self.spinBoxFigureDPI.setProperty("value", 100)
        self.spinBoxFigureDPI.setObjectName("spinBoxFigureDPI")
        self.gridLayout_2.addWidget(self.spinBoxFigureDPI, 1, 0, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem2, 8, 0, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.FigureParameters)
        self.label_9.setObjectName("label_9")
        self.gridLayout_2.addWidget(self.label_9, 0, 0, 1, 1)
        self.spinBoxFigureSavedDPI = QtWidgets.QSpinBox(self.FigureParameters)
        self.spinBoxFigureSavedDPI.setMinimum(10)
        self.spinBoxFigureSavedDPI.setMaximum(100000000)
        self.spinBoxFigureSavedDPI.setSingleStep(100)
        self.spinBoxFigureSavedDPI.setProperty("value", 100)
        self.spinBoxFigureSavedDPI.setObjectName("spinBoxFigureSavedDPI")
        self.gridLayout_2.addWidget(self.spinBoxFigureSavedDPI, 3, 0, 1, 1)
        self.frame = QtWidgets.QFrame(self.FigureParameters)
        self.frame.setFrameShape(QtWidgets.QFrame.Box)
        self.frame.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame.setObjectName("frame")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_4.setContentsMargins(1, 1, 1, 1)
        self.verticalLayout_4.setSpacing(2)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label_11 = QtWidgets.QLabel(self.frame)
        self.label_11.setObjectName("label_11")
        self.verticalLayout_4.addWidget(self.label_11)
        self.label_12 = QtWidgets.QLabel(self.frame)
        self.label_12.setObjectName("label_12")
        self.verticalLayout_4.addWidget(self.label_12)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setContentsMargins(5, 5, 5, 5)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.spinBoxFigSizeWidth = QtWidgets.QDoubleSpinBox(self.frame)
        self.spinBoxFigSizeWidth.setMaximum(100.0)
        self.spinBoxFigSizeWidth.setObjectName("spinBoxFigSizeWidth")
        self.horizontalLayout_3.addWidget(self.spinBoxFigSizeWidth)
        self.spinBoxFigSizeHeight = QtWidgets.QDoubleSpinBox(self.frame)
        self.spinBoxFigSizeHeight.setMaximum(100.0)
        self.spinBoxFigSizeHeight.setObjectName("spinBoxFigSizeHeight")
        self.horizontalLayout_3.addWidget(self.spinBoxFigSizeHeight)
        self.verticalLayout_4.addLayout(self.horizontalLayout_3)
        self.gridLayout_2.addWidget(self.frame, 4, 0, 1, 1)
        self.pushButtonApplyFigParams = QtWidgets.QPushButton(self.FigureParameters)
        self.pushButtonApplyFigParams.setObjectName("pushButtonApplyFigParams")
        self.gridLayout_2.addWidget(self.pushButtonApplyFigParams, 5, 0, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem3, 6, 0, 1, 1)
        self.toolBox.addItem(self.FigureParameters, "")
        self.AxisParameters = QtWidgets.QWidget()
        self.AxisParameters.setGeometry(QtCore.QRect(0, 0, 200, 610))
        self.AxisParameters.setObjectName("AxisParameters")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.AxisParameters)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_5 = QtWidgets.QLabel(self.AxisParameters)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_3.addWidget(self.label_5)
        self.checkBoxAxisVisible = QtWidgets.QCheckBox(self.AxisParameters)
        self.checkBoxAxisVisible.setMinimumSize(QtCore.QSize(0, 20))
        self.checkBoxAxisVisible.setText("")
        self.checkBoxAxisVisible.setChecked(True)
        self.checkBoxAxisVisible.setObjectName("checkBoxAxisVisible")
        self.verticalLayout_3.addWidget(self.checkBoxAxisVisible)
        self.label_6 = QtWidgets.QLabel(self.AxisParameters)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_3.addWidget(self.label_6)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(-1, -1, -1, 10)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lineEditXLimMin = QtWidgets.QLineEdit(self.AxisParameters)
        self.lineEditXLimMin.setObjectName("lineEditXLimMin")
        self.horizontalLayout.addWidget(self.lineEditXLimMin)
        self.lineEditXLimMax = QtWidgets.QLineEdit(self.AxisParameters)
        self.lineEditXLimMax.setObjectName("lineEditXLimMax")
        self.horizontalLayout.addWidget(self.lineEditXLimMax)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.label_7 = QtWidgets.QLabel(self.AxisParameters)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_3.addWidget(self.label_7)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setContentsMargins(-1, -1, -1, 10)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lineEditYLimMin = QtWidgets.QLineEdit(self.AxisParameters)
        self.lineEditYLimMin.setObjectName("lineEditYLimMin")
        self.horizontalLayout_2.addWidget(self.lineEditYLimMin)
        self.lineEditYLimMax = QtWidgets.QLineEdit(self.AxisParameters)
        self.lineEditYLimMax.setObjectName("lineEditYLimMax")
        self.horizontalLayout_2.addWidget(self.lineEditYLimMax)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.label_8 = QtWidgets.QLabel(self.AxisParameters)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_3.addWidget(self.label_8)
        self.comboBoxAxisScaling = QtWidgets.QComboBox(self.AxisParameters)
        self.comboBoxAxisScaling.setObjectName("comboBoxAxisScaling")
        self.comboBoxAxisScaling.addItem("")
        self.comboBoxAxisScaling.addItem("")
        self.comboBoxAxisScaling.addItem("")
        self.comboBoxAxisScaling.addItem("")
        self.comboBoxAxisScaling.addItem("")
        self.verticalLayout_3.addWidget(self.comboBoxAxisScaling)
        spacerItem4 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem4)
        self.gridLayout_3.addLayout(self.verticalLayout_3, 1, 0, 1, 1)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label = QtWidgets.QLabel(self.AxisParameters)
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label)
        self.lineEditTitle = QtWidgets.QLineEdit(self.AxisParameters)
        self.lineEditTitle.setObjectName("lineEditTitle")
        self.verticalLayout_2.addWidget(self.lineEditTitle)
        self.label_2 = QtWidgets.QLabel(self.AxisParameters)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.lineEditXLabel = QtWidgets.QLineEdit(self.AxisParameters)
        self.lineEditXLabel.setObjectName("lineEditXLabel")
        self.verticalLayout_2.addWidget(self.lineEditXLabel)
        self.label_3 = QtWidgets.QLabel(self.AxisParameters)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_2.addWidget(self.label_3)
        self.lineEditYLabel = QtWidgets.QLineEdit(self.AxisParameters)
        self.lineEditYLabel.setObjectName("lineEditYLabel")
        self.verticalLayout_2.addWidget(self.lineEditYLabel)
        self.label_4 = QtWidgets.QLabel(self.AxisParameters)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_2.addWidget(self.label_4)
        self.comboBoxAspect = QtWidgets.QComboBox(self.AxisParameters)
        self.comboBoxAspect.setObjectName("comboBoxAspect")
        self.comboBoxAspect.addItem("")
        self.comboBoxAspect.addItem("")
        self.verticalLayout_2.addWidget(self.comboBoxAspect)
        self.gridLayout_3.addLayout(self.verticalLayout_2, 0, 0, 1, 1)
        self.toolBox.addItem(self.AxisParameters, "")
        self.verticalLayout_5.addWidget(self.toolBox, 0, QtCore.Qt.AlignLeft)
        spacerItem5 = QtWidgets.QSpacerItem(20, 100, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Maximum)
        self.verticalLayout_5.addItem(spacerItem5)
        self.pushButtonDefaultView = QtWidgets.QPushButton(self.groupBox)
        self.pushButtonDefaultView.setMaximumSize(QtCore.QSize(200, 16777215))
        self.pushButtonDefaultView.setBaseSize(QtCore.QSize(200, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonDefaultView.setFont(font)
        self.pushButtonDefaultView.setObjectName("pushButtonDefaultView")
        self.verticalLayout_5.addWidget(self.pushButtonDefaultView)
        self.pushButtonClearAll = QtWidgets.QPushButton(self.groupBox)
        self.pushButtonClearAll.setMaximumSize(QtCore.QSize(200, 16777215))
        self.pushButtonClearAll.setBaseSize(QtCore.QSize(200, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonClearAll.setFont(font)
        self.pushButtonClearAll.setObjectName("pushButtonClearAll")
        self.verticalLayout_5.addWidget(self.pushButtonClearAll)
        self.verticalLayout_5.setStretch(0, 10)
        self.verticalLayout_5.setStretch(1, 10)
        self.verticalLayout_5.setStretch(2, 10)
        self.verticalLayout_5.setStretch(3, 10)
        self.horizontalLayout_4.addWidget(self.groupBox)
        self.horizontalLayout_4.setStretch(1, 10)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1208, 22))
        self.menubar.setObjectName("menubar")
        self.menuFormat = QtWidgets.QMenu(self.menubar)
        self.menuFormat.setObjectName("menuFormat")
        self.menuFigure_Width_Format = QtWidgets.QMenu(self.menuFormat)
        self.menuFigure_Width_Format.setEnabled(False)
        self.menuFigure_Width_Format.setObjectName("menuFigure_Width_Format")
        self.menuFigure_Format_Settings = QtWidgets.QMenu(self.menuFormat)
        self.menuFigure_Format_Settings.setObjectName("menuFigure_Format_Settings")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionStyle = QtWidgets.QAction(MainWindow)
        self.actionStyle.setEnabled(False)
        self.actionStyle.setObjectName("actionStyle")
        self.actionSingle_Column = QtWidgets.QAction(MainWindow)
        self.actionSingle_Column.setEnabled(False)
        self.actionSingle_Column.setObjectName("actionSingle_Column")
        self.actionDouble_Column = QtWidgets.QAction(MainWindow)
        self.actionDouble_Column.setEnabled(False)
        self.actionDouble_Column.setObjectName("actionDouble_Column")
        self.actionJournal_Styles = QtWidgets.QAction(MainWindow)
        self.actionJournal_Styles.setEnabled(False)
        self.actionJournal_Styles.setObjectName("actionJournal_Styles")
        self.actionMacro_Schema = QtWidgets.QAction(MainWindow)
        self.actionMacro_Schema.setEnabled(False)
        self.actionMacro_Schema.setObjectName("actionMacro_Schema")
        self.actionExport_Lines_to_CSV = QtWidgets.QAction(MainWindow)
        self.actionExport_Lines_to_CSV.setVisible(False)
        self.actionExport_Lines_to_CSV.setObjectName("actionExport_Lines_to_CSV")
        self.actionExport_Bars_to_CSV = QtWidgets.QAction(MainWindow)
        self.actionExport_Bars_to_CSV.setVisible(False)
        self.actionExport_Bars_to_CSV.setObjectName("actionExport_Bars_to_CSV")
        self.actionExport_Fill_Between_to_CSV = QtWidgets.QAction(MainWindow)
        self.actionExport_Fill_Between_to_CSV.setVisible(False)
        self.actionExport_Fill_Between_to_CSV.setObjectName("actionExport_Fill_Between_to_CSV")
        self.actionExport_Image_to_CSV = QtWidgets.QAction(MainWindow)
        self.actionExport_Image_to_CSV.setVisible(False)
        self.actionExport_Image_to_CSV.setObjectName("actionExport_Image_to_CSV")
        self.actionFigureDPI = QtWidgets.QAction(MainWindow)
        self.actionFigureDPI.setObjectName("actionFigureDPI")
        self.actionFigureSavedDPI = QtWidgets.QAction(MainWindow)
        self.actionFigureSavedDPI.setObjectName("actionFigureSavedDPI")
        self.actionFigure_Size_Display = QtWidgets.QAction(MainWindow)
        self.actionFigure_Size_Display.setObjectName("actionFigure_Size_Display")
        self.actionFigure_Size_Saved = QtWidgets.QAction(MainWindow)
        self.actionFigure_Size_Saved.setObjectName("actionFigure_Size_Saved")
        self.menuFigure_Width_Format.addAction(self.actionSingle_Column)
        self.menuFigure_Width_Format.addAction(self.actionDouble_Column)
        self.menuFigure_Format_Settings.addAction(self.actionFigureDPI)
        self.menuFigure_Format_Settings.addAction(self.actionFigureSavedDPI)
        self.menuFigure_Format_Settings.addAction(self.actionFigure_Size_Display)
        self.menuFigure_Format_Settings.addAction(self.actionFigure_Size_Saved)
        self.menuFormat.addAction(self.actionStyle)
        self.menuFormat.addAction(self.menuFigure_Width_Format.menuAction())
        self.menuFormat.addSeparator()
        self.menuFormat.addAction(self.menuFigure_Format_Settings.menuAction())
        self.menuFormat.addSeparator()
        self.menuFormat.addAction(self.actionJournal_Styles)
        self.menuFormat.addAction(self.actionMacro_Schema)
        self.menuFile.addAction(self.actionExport_Lines_to_CSV)
        self.menuFile.addAction(self.actionExport_Bars_to_CSV)
        self.menuFile.addAction(self.actionExport_Fill_Between_to_CSV)
        self.menuFile.addAction(self.actionExport_Image_to_CSV)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuFormat.menuAction())

        self.retranslateUi(MainWindow)
        self.toolBox.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "SciPlot PyQt"))
        self.label_10.setText(_translate("MainWindow", "DPI: Saved"))
        self.label_9.setText(_translate("MainWindow", "DPI: Display"))
        self.label_11.setText(_translate("MainWindow", "Figure Size (inches)"))
        self.label_12.setText(_translate("MainWindow", "Width x Height"))
        self.pushButtonApplyFigParams.setText(_translate("MainWindow", "Apply New Parameters"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.FigureParameters), _translate("MainWindow", "Figure Parameters"))
        self.label_5.setText(_translate("MainWindow", "Axis Visibility"))
        self.label_6.setText(_translate("MainWindow", "X Limits"))
        self.label_7.setText(_translate("MainWindow", "Y Limits"))
        self.label_8.setText(_translate("MainWindow", "Scaling"))
        self.comboBoxAxisScaling.setItemText(0, _translate("MainWindow", "auto"))
        self.comboBoxAxisScaling.setItemText(1, _translate("MainWindow", "equal"))
        self.comboBoxAxisScaling.setItemText(2, _translate("MainWindow", "tight"))
        self.comboBoxAxisScaling.setItemText(3, _translate("MainWindow", "image"))
        self.comboBoxAxisScaling.setItemText(4, _translate("MainWindow", "square"))
        self.label.setText(_translate("MainWindow", "Title"))
        self.label_2.setText(_translate("MainWindow", "X Label"))
        self.label_3.setText(_translate("MainWindow", "Y Label"))
        self.label_4.setText(_translate("MainWindow", "Aspect Ratio"))
        self.comboBoxAspect.setItemText(0, _translate("MainWindow", "auto"))
        self.comboBoxAspect.setItemText(1, _translate("MainWindow", "equal"))
        self.toolBox.setItemText(self.toolBox.indexOf(self.AxisParameters), _translate("MainWindow", "Axis Parameters"))
        self.pushButtonDefaultView.setText(_translate("MainWindow", "Set Default View"))
        self.pushButtonClearAll.setText(_translate("MainWindow", "Clear All"))
        self.menuFormat.setTitle(_translate("MainWindow", "Format"))
        self.menuFigure_Width_Format.setTitle(_translate("MainWindow", "Figure Width Format"))
        self.menuFigure_Format_Settings.setTitle(_translate("MainWindow", "Figure Format Settings"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.actionStyle.setText(_translate("MainWindow", "Color Scheme"))
        self.actionSingle_Column.setText(_translate("MainWindow", "Single-Column"))
        self.actionDouble_Column.setText(_translate("MainWindow", "Double-Column"))
        self.actionJournal_Styles.setText(_translate("MainWindow", "Journal Styles"))
        self.actionMacro_Schema.setText(_translate("MainWindow", "Macro Schema"))
        self.actionExport_Lines_to_CSV.setText(_translate("MainWindow", "Export Lines to CSV"))
        self.actionExport_Bars_to_CSV.setText(_translate("MainWindow", "Export Bars to CSV"))
        self.actionExport_Fill_Between_to_CSV.setText(_translate("MainWindow", "Export Fill Between to CSV"))
        self.actionExport_Image_to_CSV.setText(_translate("MainWindow", "Export Image to CSV"))
        self.actionFigureDPI.setText(_translate("MainWindow", "DPI: Display"))
        self.actionFigureSavedDPI.setText(_translate("MainWindow", "DPI: Saved"))
        self.actionFigure_Size_Display.setText(_translate("MainWindow", "Figure Size: Display"))
        self.actionFigure_Size_Saved.setText(_translate("MainWindow", "Figure Size: Saved"))





# -*- coding: utf-8 -*-
"""
Generic data containers

Created on Thu Jul  7 10:31:43 2016

@author: chc
"""

import numpy as _np

class Data:
    """
    Contains the underlying data to be plotted
    
    Attributes
    ----------
    x : ndarray (1D)
        x-data (effectively, the horizontal axis data)
    
    y : ndarray (1D)
        y-data (effectively, the vertical axis data)
    
    label : str
        Label information, i.e., the name of the plot, graph, etc.
        
    style_dict : dict
        Style information (e.g., color, linewidth, etc)
        
    mplobj : object
        Object returned from the MPL plotting procedure.
    """
    def __init__(self):
        self._setupData()

    def _setupData(self):
        self.x = None
        self.y = None
        self.id = None
        self.label = None
        self.mplobj = None
        #self.units = {'x_units': None, 'y_units': None}
        self.style_dict = {}
        self.meta = {}

    @property
    def model_style(self):
        out = {}
        out.update(self.style_dict)
        out['label'] = self.label
        out['id'] = self.id
        out['meta'] = self.meta
        return out

class Data2D(Data):
    """
    Contains the underlying data to be imaged
    """
    def __init__(self):
        self._setupData()
        self.img = None

    @property
    def extent(self):
        """
        Generate an extent list used for mpl.imshow
        [xmin, xmax, ymin, ymax]
        """
        ext = [None, None, None, None]

        if self.x is not None:
            ext[0] = self.x.min()
            ext[1] = self.x.max()
        else:
            ext[0] = 0
            ext[1] = self.img.shape[1]

        if self.y is not None:
            ext[2] = self.y.min()
            ext[3] = self.y.max()
        else:
            ext[2] = 0
            ext[3] = self.img.shape[0]

        return ext

class DataGlobal:
    """
    Contains data that is pertinent across all plots
    """
    def __init__(self):
        self.labels = {'x_label': None, 'y_label': None, 'title': None}



# -*- coding: utf-8 -*-
"""
Created on Fri Jul  8 13:52:57 2016

@author: chc
"""

from sciplot.data.generic import Data2D as _Data2D
import matplotlib as _mpl
import numpy as _np


class ImageStyle:
    """
    Style information for each data plotted (images)
    """
    def __init__(self):
        self._setupImageStyle()

    def _setupImageStyle(self):
        """
        Style dictioonary.

        Maybe include cmap color in the future
        """
        self.style_dict = {'cmap_name': None,
                           'alpha': None,
                           'clim': None}

    def retrieve_style_from_image(self, image):
        """
        Take an MPL image object and retrieve appropriate attributes
        """

        # cmap name
        cmap_name = image.get_cmap().name
        if cmap_name is None:
            cmap_name = 'Custom'
        self.style_dict['cmap_name'] = cmap_name

        # Alpha (transparency)
        alpha = image.get_alpha()
        if alpha is None:
            alpha = 1
        self.style_dict['alpha'] = alpha

        # clim
        self.style_dict['clim'] = image.get_clim()


class DataImages(_Data2D, ImageStyle):
    def __init__(self):
        self.cbar = {'obj' : None, 'show' : False}
        self._setupData()
        self._setupImageStyle()

    @property
    def model_style(self):
        out = {}
        out.update(self.style_dict)
        
        # clim broken out into high and low in model
        clim = out.pop('clim')
        out['clim_low'] = _np.min(clim)
        out['clim_high'] = _np.max(clim)
        
        out['label'] = self.label
        out['id'] = self.id
        out['meta'] = self.meta
        out['colorbar'] = self.cbar['show']
        return out

    @model_style.setter
    def model_style(self, value):
        self.label = value['label']
        self.meta = value['meta']
        self.style_dict['cmap_name'] = value['cmap_name']
        self.style_dict['alpha'] = value['alpha']

        # clim broken out into high and low in model
        self.style_dict['clim'] = list((value['clim_low'], value['clim_high']))
        
        # colobar
        self.cbar['show'] = value['colorbar']



"""

"""

import sys as _sys
import os as _os
from PyQt5.QtWidgets import (QApplication as _QApplication)
# from crikit import CRIkitUI
from sciplot import sciplotUI

def main(args=None):
    """
    The main routine.
    """

    if args is None:
        args = _sys.argv[1:]

    print('Starting up sciplot...')

    app = _QApplication(_sys.argv)
    app.setStyle('Cleanlooks')
    win = sciplotUI.SciPlotUI() ### EDIT ###

    win.showMaximized()

    app.exec_()

if __name__ == '__main__':
    _sys.exit(main())


# -*- coding: utf-8 -*-
"""
Created on Thu Jul  7 10:30:28 2016

@author: chc
"""

from sciplot.data.generic import Data as _Data
import matplotlib as _mpl


class BarStyle:
    """
    Style information for each data plotted (bar)
    """
    def __init__(self):
        self._setupBarStyle()

    def _setupBarStyle(self):
        self.style_dict = {'facecolor': None,
                           'alpha': None,
                           'edgecolor': None,
                           'linewidth': None,
                           'width_factor': None}

    def retrieve_style_from_bar(self, bar):
        """
        Take an MPL bar object and retrieve appropriate attributes
        """

        # facecolor
        facecolor = bar.get_facecolor()[:-1]
        if isinstance(facecolor, str):
            # facecolor = _mpl.colors.ColorConverter.cache[facecolor]
            facecolor = _mpl.colors.ColorConverter().to_rgb(facecolor)
        if isinstance(facecolor, tuple):
            facecolor = list(facecolor)
        self.style_dict['facecolor'] = facecolor

        # Alpha (transparency)
        alpha = bar.get_alpha()
        if alpha is None:
            alpha = 1
        self.style_dict['alpha'] = alpha

        # edgecolor
        edgecolor = bar.get_edgecolor()[:-1]
        if isinstance(edgecolor, str):
            # edgecolor = _mpl.colors.ColorConverter.cache[edgecolor]
            edgecolor = _mpl.colors.ColorConverter().to_rgb(edgecolor)
        if isinstance(edgecolor, tuple):
            edgecolor = list(edgecolor)
        self.style_dict['edgecolor'] = edgecolor

        # Linewidth
        self.style_dict['linewidth'] = bar.get_linewidth()


class DataBar(_Data, BarStyle):
    def __init__(self):
        self._width = None
        self._left = None
        self._gap = None
        self._bottom = None
        self._setupData()
        self._setupBarStyle()

    @property
    def model_style(self):
        out = {}
        out.update(self.style_dict)
        out['label'] = self.label
        out['id'] = self.id
        out['meta'] = self.meta
        return out

    @model_style.setter
    def model_style(self, value):
        self.label = value['label']
        self.meta = value['meta']
        self.style_dict['facecolor'] = value['facecolor']
        self.style_dict['alpha'] = value['alpha']
        self.style_dict['edgecolor'] = value['edgecolor']
        self.style_dict['linewidth'] = value['linewidth']
        self.style_dict['width_factor'] = value['width_factor']
        if self._gap is not None:
            self._width = self._gap*self.style_dict['width_factor']
        else:
            self._width = self.style_dict['width_factor']
        self._left = self.x - self._width/2



# -*- coding: utf-8 -*-
"""
Created on Thu Jun 30 10:23:03 2016

@author: chc
"""

from cycler import cycler
import copy as _copy
import numpy as _np
import matplotlib as _mpl
import matplotlib.markers
import matplotlib.lines

class MplMarkers:
    MARKER_DICT = _mpl.markers.MarkerStyle.markers
    MARKER_DESC = []
    MARKER_SYMBOL = []
    for count in MARKER_DICT:
        MARKER_SYMBOL.append(count)
        MARKER_DESC.append(MARKER_DICT[count])

    @classmethod
    def index(cls, str_in):
        idx = None
        try:
            idx = cls.MARKER_DESC.index(str_in)
        except:
            idx = None
            try:
                idx = cls.MARKER_SYMBOL.index(str_in)
            except:
                idx = None
        return idx

class MplLines:
    LINESTYLE_DICT = _mpl.lines.lineStyles
    LINESTYLE_DESC = []
    LINESTYLE_SYMBOL = []
    for count in LINESTYLE_DICT:
        LINESTYLE_SYMBOL.append(count)
        LINESTYLE_DESC.append(LINESTYLE_DICT[count])

    @classmethod
    def index(cls, str_in):
        idx = None
        try:
            idx = cls.LINESTYLE_DESC.index(str_in)
        except:
            idx = None
            try:
                idx = cls.LINESTYLE_SYMBOL.index(str_in)
            except:
                idx = None
        return idx

class MplStyleSheets:
    """

    """

    _tableau20 = _np.array([[31, 119, 180],
                           [174, 199, 232],
                           [255, 127, 14],
                           [255, 187, 120],
                           [44, 160, 44],
                           [152, 223, 138],
                           [214, 39, 40],
                           [255, 152, 150],
                           [148, 103, 189],
                           [197, 176, 213],
                           [140, 86, 75],
                           [196, 156, 148],
                           [227, 119, 194],
                           [247, 182, 210],
                           [127, 127, 127],
                           [199, 199, 199],
                           [188, 189, 34],
                           [219, 219, 141],
                           [23, 190, 207],
                           [158, 218, 229]], dtype=_np.float)/255

    _tableau10 = _tableau20[::2,:]

    _tableau10_med = _np.array([[114, 158, 206],
                                [255, 158, 74],
                                [103, 191, 92],
                                [237, 102, 93],
                                [173, 139, 201],
                                [168, 120, 110],
                                [237, 151, 202],
                                [162, 162, 162],
                                [205, 204, 93],
                                [109, 204, 218]], dtype = _np.float)/255


    _base_crikit = {'font.family': ['sans-serif'],
                    'font.sans-serif': ['Arial',
                                        'Bitstream Vera Sans',
                                        'DejaVu Sans',
                                        'Lucida Grande',
                                        'Verdana',
                                        'Geneva',
                                        'Lucid',
                                        'Avant Garde',
                                        'sans-serif'],
                    'axes.prop_cycle': cycler('color', _tableau10),
#                    'axes.prop_cycle': plt.style.library['ggplot']['axes.prop_cycle'],
                    'image.cmap': 'viridis',
                    'image.interpolation': 'none'}

    _paper_halfwidth = {'axes.labelsize': 8.8,
                 'axes.titlesize': 9.6,
                 'figure.figsize': [3.3, 4.4],
                 'grid.linewidth': 0.8,
                 'legend.fontsize': 8.0,
                 'lines.linewidth': 1.4,
                 'lines.markeredgewidth': 0.0,
                 'lines.markersize': 5.6,
                 'patch.linewidth': 0.24,
                 'xtick.labelsize': 8.0,
                 'xtick.major.pad': 5.6,
                 'xtick.major.width': 0.8,
                 'xtick.minor.width': 0.4,
                 'ytick.labelsize': 8.0,
                 'ytick.major.pad': 5.6,
                 'ytick.major.width': 0.8,
                 'ytick.minor.width': 0.4}

    _paper_fullwidth = {'axes.labelsize': 8.8,
                 'axes.titlesize': 9.6,
                 'figure.figsize': [6.4, 4.4],
                 'grid.linewidth': 0.8,
                 'legend.fontsize': 8.8,
                 'lines.linewidth': 1.4,
                 'lines.markeredgewidth': 0.0,
                 'lines.markersize': 5.6,
                 'patch.linewidth': 0.24,
                 'xtick.labelsize': 8.0,
                 'xtick.major.pad': 5.6,
                 'xtick.major.width': 0.8,
                 'xtick.minor.width': 0.4,
                 'ytick.labelsize': 8.0,
                 'ytick.major.pad': 5.6,
                 'ytick.major.width': 0.8,
                 'ytick.minor.width': 0.4}

    _poster = {'axes.labelsize': 17.6,
               'axes.titlesize': 19.2,
               'figure.figsize': [12.8, 8.8],
              'grid.linewidth': 1.6,
              'legend.fontsize': 16.0,
              'lines.linewidth': 2.8,
              'lines.markeredgewidth': 0.0,
              'lines.markersize': 11.2,
              'patch.linewidth': 0.48,
              'xtick.labelsize': 16.0,
              'xtick.major.pad': 11.2,
              'xtick.major.width': 1.6,
              'xtick.minor.width': 0.8,
              'ytick.labelsize': 16.0,
              'ytick.major.pad': 11.2,
              'ytick.major.width': 1.6,
              'ytick.minor.width': 0.8}

    basic_halfwidth = _copy.deepcopy(_base_crikit)
    basic_halfwidth.update(_paper_halfwidth)

    basic_fullwidth = _copy.deepcopy(_base_crikit)
    basic_fullwidth.update(_paper_fullwidth)

    basic_poster = _copy.deepcopy(_base_crikit)
    basic_poster.update(_poster)


if __name__ == '__main__':
    import matplotlib.pyplot as _plt
    import numpy as _np

    x = _np.arange(100)
    _plt.style.use('classic')
    style = MplStyleSheets.basic_fullwidth
    _plt.style.use(style)

    _plt.figure()

    _plt.plot((_np.random.rand(10,1)*x).T, label='test')
    _plt.legend()
    _plt.show()

    _plt.figure()
    _plt.imshow(_np.random.rand(100,100))
    _plt.colorbar()
    _plt.show()



"""
Test utility functions
"""
from sciplot.utils.general import round_list

def test_round_list():
    list_under_test = [0.111, 0.222, 0.555]

    list_round_3 = [0.111, 0.222, 0.555]
    list_round_2 = [0.11, 0.22, 0.56]

    assert round_list(list_under_test, 3) == list_round_3
    assert round_list(list_under_test, 2) == list_round_2


# -*- coding: utf-8 -*-
"""

Demonstration of adding functionality to Sciplot

Created on Mon Jul 11 23:46:01 2016

@author: chc
"""

import numpy as _np

# Append sys path
import sys as _sys
import os as _os
if __name__ == '__main__':
    _sys.path.append(_os.path.abspath('../'))

from PyQt5.QtWidgets import (QApplication as _QApplication,
                             QTabWidget as _QTabWidget,
                             QCalendarWidget as _QCalendarWidget)

from sciplot.sciplotUI import SciPlotUI as _SciPlotUI

class SciPlotAddon(_SciPlotUI):
    """
    Trivial example of using SciPlot with addons (or building another \
    application on top of SciPlot).
    """
    def __init__(self, limit_to=None, parent=None):
        super(SciPlotAddon, self).__init__(limit_to=limit_to, parent=parent, show=True)
        # Setup SciPlot window
        # self.setup(limit_to=limit_to, parent=parent)

        # Add a calendar to the tabs and have it displayed initially
        cal1 = _QCalendarWidget()
        self.ui.modelTabWidget.insertTab(0, cal1, 'Add-On Tab')
        self.ui.modelTabWidget.setCurrentIndex(0)

        # Add a calendar to the toolBox and have it displayed initially
        cal2 = _QCalendarWidget()
        self.ui.toolBox.addItem(cal2, 'Add-On Toolbox Tab')
        self.ui.toolBox.setCurrentIndex(self.ui.toolBox.count()-1)

        # Adjust with of groupBox to accomodate new widget (10% larger than)
        width = cal2.sizeHint().width()
        self.ui.groupBox.setMinimumWidth(1.1*width)
        self.ui.groupBox.setMaximumWidth(1.1*width)
        self.ui.groupBox.updateGeometry()

if __name__ == '__main__':

    app = _QApplication(_sys.argv)

    winPlotter = SciPlotAddon(limit_to=['bars'])

    winPlotter.show()

    x = _np.arange(100)
    y = x**2

#    winPlotter.plot(x, y, x_label='X', label='Plot')
#    winPlotter.plot(x, y**1.1, label='Plot 2')
#    winPlotter.fill_between(x, y-1000, y+1000, label='Fill Between')
#
#    winPlotter.imshow(_np.random.randn(100,100), label='Imshow')
#    winPlotter.bar(x[::10],y[::10],label='Bar')
#    winPlotter.hist(y,label='Hist')

    winPlotter.bar(0,10, label='Bar: single-value')
    _sys.exit(app.exec_())



"""
Just a basic test
"""
import sys
import numpy as np

from PyQt5.QtWidgets import QApplication

from sciplot.sciplotUI import SciPlotUI

def test_pass():
    pass
    # app = QApplication(sys.argv)
    # winPlotter = SciPlotUI(limit_to=['lines','bars', 'fill betweens',
    #                                  'images'])
    
    # x = np.arange(100)
    # y = x**2
    # winPlotter.plot(x, y, x_label='X', label='Plot')
    # winPlotter.fill_between(x, y-1000, y+1000, label='Fill Between')
    # winPlotter.imshow(np.random.randn(100,100), label='Imshow', cbar=True)
    # winPlotter.bar(x[::10],y[::10],label='Bar')
    # app.exec_()


# -*- coding: utf-8 -*-
"""
Create generic MPL canvas, toolbar, figure, axis.

Heavily borrowed from:
    matplotlib.org/examples/user_interfaces/embedding_in_qt5.html

Created on Thu Jun 30 15:41:35 2016

@author: chc
"""

import sys as _sys
import os as _os
import numpy as _np
import matplotlib as _mpl
import matplotlib.style as _mpl_sty

from PyQt5 import QtWidgets as _QtWidgets

from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as \
    FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as \
    _NavigationToolbar

from matplotlib.figure import Figure as _Figure

# Make sure that we are using QT5
_mpl.use('Qt5Agg')


class MplCanvas(FigureCanvas):
    """Ultimately, this is a QWidget (as well as a FigureCanvasAgg, etc.)."""

    def __init__(self, subplot=111, parent=None, width=5, height=4, dpi=100, 
                 figfacecolor=3*[0.941], style=None, **kwargs):

        #        super(MplCanvas, self).__init__(parent)
        if style is None:
            pass
        else:
            _mpl.style.use(style)
        self._mpl_v1 = _mpl.__version__.split('.')[0] == 1

        # Create figure and axes
        self.fig = _Figure(figsize=(width, height), dpi=dpi, 
                           facecolor=figfacecolor, **kwargs)
        # Initialize the canvas
        FigureCanvas.__init__(self, self.fig)
        self.setParent(parent)
        
        self.setupAx(subplot=subplot, **kwargs)
        
        # Not really used, but could be used to have some sort of initial plot
        self.compute_initial_figure()

        # Set canvas size policies and geometry
        FigureCanvas.setSizePolicy(self, _QtWidgets.QSizePolicy.Fixed,
                                   _QtWidgets.QSizePolicy.Fixed)
        FigureCanvas.updateGeometry(self)

        # Create the toolbar and connect to canvas (self)
        self.toolbar = _NavigationToolbar(self, None)
        
    def setupAx(self, subplot=111, **kwargs):
        if subplot > 339:
            raise ValueError('Subplot is limited to 3x3 or less')
        # Break out e.g. 111 to [1, 1, 1]
        subplot_list = [int(val) for val in list(str(subplot))]
        
        # Total number of subplots
        num_plots = subplot_list[0]*subplot_list[1]
        
        # Num of vertical subplots
        vnum = subplot_list[0]

        # Num of horiz subplots
        hnum = subplot_list[1]

        # [start, stop) count
        c_start = vnum*100 + hnum*10 + 1
        c_stop = vnum*100 + hnum*10 + num_plots + 1
        
        if num_plots <= 0:
            raise ValueError('subplot need be 1x1 to 3x3')
        elif num_plots == 1:
            self.ax = self.fig.add_subplot(111, **kwargs)
        elif num_plots > 1:
            self.ax = []
            for num, splt_num in enumerate(_np.arange(c_start, c_stop)):
                self.ax.append(self.fig.add_subplot(splt_num, **kwargs))
                if self._mpl_v1:
                    self.ax[num].hold(False)
            self.fig.tight_layout()

    def compute_initial_figure(self):
        pass


if __name__ == '__main__':

    from PyQt5 import QtCore as _QtCore

    class ApplicationWindow(_QtWidgets.QMainWindow):
        def __init__(self, subplot=111, style=None):
            _QtWidgets.QMainWindow.__init__(self)
            #super(ApplicationWindow, self).__init__(None)
            self.setAttribute(_QtCore.Qt.WA_DeleteOnClose)

            self.main_widget = _QtWidgets.QWidget(self)

            self.mpl_layout = _QtWidgets.QVBoxLayout(self.main_widget)
            self.mpl_widget = MplCanvas(parent=self.main_widget, 
                                        subplot=subplot, width=5, height=4,
                                        dpi=100, style=style)
            self.mpl_layout.addWidget(self.mpl_widget.toolbar)
            self.mpl_layout.addWidget(self.mpl_widget)

            self.main_widget.setFocus()
            self.setCentralWidget(self.main_widget)

            self.setSizePolicy(_QtWidgets.QSizePolicy.Expanding,
                               _QtWidgets.QSizePolicy.Expanding)

    qApp = _QtWidgets.QApplication(_sys.argv)
 
    
    aw2 = ApplicationWindow(style='seaborn-deep', subplot=211)
    aw2.mpl_widget.ax[0].set_title('0')
    aw2.mpl_widget.ax[1].set_title('1')
    aw2.show()

    qApp.exec_()



# -*- coding: utf-8 -*-
"""
SciPlot-PyQt: Publication-ready scientific plotting for Python
==============================================================

SciPlot-PyQt (aka SciPlot) is a user-interface/matplotlib wrapper built with 
PyQt5 that allows interactive plotting through an embedded matplotlib canvas. 
It enables fast and easy publication-ready plots and images:
    
    * Interactive plotting
    * Theme and style editing (TODO)
    * Figure saving and opening for later editing (TODO)
    
Supported Plot Types
---------------------
Line plots : plot

Bar plots : bar, hist

Polycollections : fill_between

Images : imshow
"""

import sys as _sys
import os as _os
import pkg_resources as _pkg_resources

from . import sciplotUI

_sys.path.append(_os.path.abspath('../'))

try:
    __version__ = _pkg_resources.require("sciplot-pyqt")[0].version
except:
    pass
    
# __version__ = '0.2.2'

# __all__ = ['SciPlotUI']

# from .sciplotUI import SciPlotUI as main



# -*- coding: utf-8 -*-
"""
Somewhat specialized data containers.

Created on Thu Jul  7 15:21:29 2016

@author: chc
"""

from sciplot.data.polycollections import PolyCollectionStyle
from sciplot.data.generic import Data as _Data

class DataFillBetween(_Data, PolyCollectionStyle):
    def __init__(self):
        self._setupData()
        self._setupPolyCollectionStyle()
        del self.y
        self.y_low = None
        self.y_high = None

    @property
    def model_style(self):
        out = {}
        out.update(self.style_dict)
        out['label'] = self.label
        out['id'] = self.id
        out['meta'] = self.meta
        return out

    @model_style.setter
    def model_style(self, value):
        self.label = value['label']
        self.meta = value['meta']
        self.style_dict['facecolor'] = value['facecolor']
        self.style_dict['alpha'] = value['alpha']
        self.style_dict['edgecolor'] = value['edgecolor']
        self.style_dict['linewidth'] = value['linewidth']



# -*- coding: utf-8 -*-
"""
Created on Thu Jul 21 01:11:04 2016

@author: chc
"""

from setuptools import setup, find_packages

with open('README.rst', encoding='utf-8') as f:
    long_description = f.read()

setup(name='sciplot-pyqt',
      version = '0.2.3',
      description = 'A small matplotlib wrapper/UI for creating \
                     publication-ready plots, graphs, and images',
      long_description = long_description,
      url = 'http://github.com/CCampJr/SciPlot-PyQt',
      author = 'Charles H. Camp Jr.',
      author_email = 'charles.camp@nist.gov',
      license = 'Public Domain',
      packages = find_packages(),
      zip_safe = False,
      include_package_data = True,
      install_requires=['numpy','matplotlib'],
      setup_requires=['pytest-runner'],
      tests_require=['pytest'],
      classifiers=['Development Status :: 3 - Alpha',
                   'Intended Audience :: Developers',
                   'Intended Audience :: Science/Research',
                   'Operating System :: OS Independent',
                   'Environment :: X11 Applications :: Qt',
                   'Programming Language :: Python :: 3 :: Only',
                   'Programming Language :: Python :: 3.4',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6',
                   'Topic :: Scientific/Engineering :: Visualization'])


# -*- coding: utf-8 -*-
"""

Abstract ModelViewDelegate for MPL objects

Created on Thu Jul  7 10:01:23 2016

@author: chc
"""

from PyQt5.QtWidgets import (QApplication as _QApplication,
                             QMainWindow as _QMainWindow,
                             QColorDialog as _QColorDialog,
                             QDoubleSpinBox as _QDoubleSpinBox,
                             QComboBox as _QComboBox,
                             QLineEdit as _QLineEdit,
                             QStyledItemDelegate as _QStyledItemDelegate,
                             QTableView as _QTableView,
                             QSizePolicy as _QSizePolicy)

from PyQt5.QtCore import (QAbstractTableModel as _QAbstractTableModel,
                          QVariant as _QVariant,
                          QObject as _QObject,
                          pyqtSignal as _pyqtSignal,
                          QModelIndex as _QModelIndex,
                          Qt as _Qt)


class AbstractTableModelMpl(_QAbstractTableModel, _QObject):

    def rowCount(self, parent=_QModelIndex()):
        return len(self._model_data)

    def columnCount(self, parent=_QModelIndex()):
        return len(self.headers)

    def headerData(self, col, orientation, role):
        if orientation == _Qt.Horizontal and role == _Qt.DisplayRole:
            return self.headers[col]
        return _QVariant()

    def doubleClickCheck(self, index):
        raise NotImplementedError

    def deleteData(self, index):
            self.setData(index, True)

    def data(self, index, role=_Qt.DisplayRole):
        raise NotImplementedError

    def setData(self, index, value, role=_Qt.EditRole):
        raise NotImplementedError

    def flags(self, index):
            flag = super(_QAbstractTableModel, self).flags(index)
            return flag | _Qt.ItemIsEditable


class AbstractEditDelegateMpl(_QStyledItemDelegate):
    def createEditor(self, parent, option, index):
        raise NotImplementedError

    def setEditorData(self, editor, index):
        raise NotImplementedError

    def setModelData(self, editor, model, index):
        raise NotImplementedError



# -*- coding: utf-8 -*-
"""
Created on Thu Jul  7 10:30:28 2016

@author: chc
"""

from sciplot.data.generic import Data as _Data
import matplotlib as _mpl


class LineStyle:
    """
    Style information for each data plotted (line)
    """
    def __init__(self):
        self._setupLineStyle()

    def _setupLineStyle(self):
        self.style_dict = {'color': None,
                           'alpha': None,
                           'linewidth': None,
                           'linestyle': None,
                           'marker': None,
                           'markersize': None}

    def retrieve_style_from_line(self, line):
        """
        Take an MPL line object and retrieve appropriate attributes
        """

        # Line color
        color = line.get_color()
        if isinstance(color, str):
            # print('Color: {}'.format(color))
            # color = _mpl.colors.ColorConverter.cache[color]
            color = _mpl.colors.ColorConverter().to_rgb(color)
        if isinstance(color, tuple):
            color = list(color)
        self.style_dict['color'] = color

        # Alpha (transparency)
        alpha = line.get_alpha()
        if alpha is None:
            alpha = 1
        self.style_dict['alpha'] = alpha

        # Linewidth
        self.style_dict['linewidth'] = line.get_linewidth()

        # Linestyle
        self.style_dict['linestyle'] = line.get_linestyle()

        # Marker
        self.style_dict['marker'] = line.get_marker()

        # Marker Size
        self.style_dict['markersize'] = line.get_markersize()


class DataLine(_Data, LineStyle):
    def __init__(self):
        self._setupData()
        self._setupLineStyle()

    @property
    def model_style(self):
        out = {}
        out.update(self.style_dict)
        out['label'] = self.label
        out['id'] = self.id
        out['meta'] = self.meta
        return out

    @model_style.setter
    def model_style(self, value):
        self.label = value['label']
        self.meta = value['meta']
        self.style_dict['color'] = value['color']
        self.style_dict['alpha'] = value['alpha']
        self.style_dict['linewidth'] = value['linewidth']
        self.style_dict['linestyle'] = value['linestyle']
        self.style_dict['marker'] = value['marker']
        self.style_dict['markersize'] = value['markersize']

class PlotsDataContainer:
    """
    Contains all plot data
    """
    def __init__(self):
        self.line_data_list = []
        self.patch_data_list = []



""" Just some general utilities """

def round_list(input, ndigits=3):
    """ Takes in a list of numbers and rounds them to a particular number of digits """
    return [round(i, ndigits) for i in input]


# -*- coding: utf-8 -*-
"""
SciPlot-PyQt: Publication-ready scientific plotting for Python
==============================================================

SciPlot-PyQt (aka SciPlot) is a user-interface/matplotlib wrapper built with 
PyQt5 that allows interactive plotting through an embedded matplotlib canvas. 
It enables fast and easy publication-ready plots and images:
    
    * Interactive plotting
    * Theme and style editing (TODO)
    * Figure saving and opening for later editing (TODO)
    
Supported Plot Types
---------------------
Line plots : plot

Bar plots : bar, hist

Polycollections : fill_between

Images : imshow

Notes
-----
SciPlot has a lot of advances/improvements to make. Feel free to contact me--
help is always welcome!

Usage
-----
import sciplot
sp = sciplot.SciPlotUI()
sp.show()

Example
-------
sp.plot((0,1),(2,3),label='Line', x_label='X', y_label='Y', ls='--')
sp.fill_between((0,1),(1,2),(3,4),label='Fill Between', color='r', alpha=0.25)

Authors
-------
* Charles H. Camp Jr. <charles.camp@nist.gov>
"""

import sys as _sys
import os as _os
import numpy as _np
import time as _time

# Generic imports for MPL-incorporation
import matplotlib as _mpl
_mpl.use('Qt5Agg')
import matplotlib.pyplot as _plt  # Simply for it starting a QApplication

# Generic imports for QT-based programs
from PyQt5.QtWidgets import (QApplication as _QApplication,
                             QMainWindow as _QMainWindow,
                             QTableView as _QTableView,
                             QSizePolicy as _QSizePolicy,
                             QTabWidget as _QTabWidget,
                             QFileDialog as _QFileDialog,
                             QInputDialog as _QInputDialog)

from PyQt5.QtCore import pyqtSignal as _pyqtSignal
import PyQt5.QtCore as _QtCore

import sciplot

# Import from Designer-based GUI
from sciplot.ui.qt_Plotter import Ui_MainWindow as Ui_Plotter
from sciplot.ui.widget_mpl import MplCanvas as _MplCanvas
from sciplot.ui.dialogs import DualEntry

from sciplot.ui.models.lines import (TableModelLines as _TableModelLines,
                                     EditDelegateLines as _EditDelegateLines)

from sciplot.ui.models.fillbetween import (TableModelFillBetween as
                                               _TableModelFillBetween,
                                           EditDelegateFillBetween as
                                           _EditDelegateFillBetween)

from sciplot.ui.models.images import (TableModelImages as _TableModelImages,
                                     EditDelegateImages as _EditDelegateImages)

from sciplot.ui.models.bars import (TableModelBars as _TableModelBars,
                                     EditDelegateBars as _EditDelegateBars)

from sciplot.data.generic import DataGlobal as _DataGlobal
from sciplot.data.lines import DataLine as _DataLine
from sciplot.data.images import DataImages as _DataImages
from sciplot.data.bars import DataBar as _DataBar
from sciplot.data.special import DataFillBetween as _DataFillBetween

class SciPlotUI(_QMainWindow):
    """
    Scientific plotting user-interface for creating publication-quality plots
    and images

    Parameters
    ----------

    limit_to : list, optional (default = None)
        Limit the application to implement only certain functionality. \
        Default is all elements turned ON. See Notes for options.
        
    show : bool, optional (default = True)
        Whether to show the UI upon instantiation

    Methods
    -------
    plot : MPL-like plotting functionality

    imshow : MPL-like imshow

    bar : MPL-like bar plot EXCEPT centered (rather than left-edge defined)

    hist : MPL-like histogram

    fill_between : MPL-like fill_between

    Internal Methods
    ----------------
    updatePlotDataStyle : Make updates to plots (lines) when a stylistic \
    change is made within the model-table

    updatePlotDataDelete : Remove a plot when deleted from model-table

    updateFillBetweenDataStyle : Make updates to fill between's when a \
    stylistic change is made within the model-table

    updateFillBetweenDataDelete : Remove a fill between when deleted from \
    model-table

    updateImagesDataStyle : Make updates to images when a stylistic \
    change is made within the model-table

    updateImageDataDelete : Remove an image when deleted from model-table

    updateBarsDataStyle : Make updates to bars plots when a stylistic \
    change is made within the model-table

    updateBarsDataDelete : Remove a bar plot when deleted from model-table

    refreshAllPlots : Delete all plots and re-plot

    updateAllLabels : Update all labels (x-, y-, title, etc) on MPL widget, \
        in model, in data container, and in UI lineEdits

    updateLineEditLabels : Update all labels (x-, y-, title, etc) in UI \
        lineEdits

    updateDataLabels : Update all labels (x-, y-, title, etc) in data \
    container

    updateMplLabels : Update all labels (x-, y-, title, etc) on MPL widget

    updateLabelsFromLineEdit : Update all labels (x-, y-, title, etc) on MPL \
        widget, in model, and in data container. Edits came from lineEdits.

    axisAspect : Set MPL-axis aspect ratio setting

    axisScaling : Set MPL-axis scaling ratio setting

    axisVisible : Set MPL-axis on or off

    axisLimits : Set MPL-axis limits

    updateAxisParameters : Query and update UI lineEdits related to axis \
        properties such as limits, visibility (on/off), scaling, and aspect \
        ratio

    Notes
    -----
    * limit_to options: 'lines', 'fill betweens', 'bars', images'
    """
    # Signal emitted when clearAll is called
    # Added for external programs
    all_cleared = _pyqtSignal(int)  
    
    def __init__(self, limit_to=None, parent=None, show=True):
        self.__version__ = sciplot.__version__
        self.list_ids = []
        self.list_all = []

        # There are a number of changes and deprectaion
        # in MPL v2 and v3 ; thus, this will be tracked
        # so MPL 1 and 2 can be used seemlessly
        self._mpl_v1 = int(_mpl.__version__.rsplit('.')[0]) == 1

        # Check to see if QApp already exists
        # if not, one has to be created
        self.app = None
        if _QApplication.instance() is None:
            print('\nNo QApplication instance (this is common with certain \
            version of Matplotlib). Creating one.\n\r\
            You will need to exec manually after you finish plotting.\n\
            -----------Example---------------\n\
            import sciplot\n\
            sp = sciplot.main()\n\n\
            # Plot a line\n\
            sp.plot((0,1),(0,1))\n\n\
            # Start the QApplication\n\
            sp.app.exec_()')
            self.app = _QApplication(_sys.argv)
            self.app.setQuitOnLastWindowClosed(True)

        self.setup(limit_to=limit_to, parent=parent)
        if show:
            self.show()
       
    def closeEvent(self, event):
        pass

    def _tabAvailability(self, limit_to=None):
        """
        If limit_to is provided, limits the tabs (elements) that are available.

        May be useful for built-upon applications.
        """
        if limit_to is None:
            self.elements = ['lines', 'fill betweens', 'images', 'bars']
            self._to_setup = [self.setupLines, self.setupFillBetweens,
                              self.setupImages, self.setupBars]
        else:
            self._to_setup = []
            self.elements = []
            if limit_to.count('lines'):
                self.elements.append('lines')
                self._to_setup.append(self.setupLines)
            if limit_to.count('fill betweens'):
                self.elements.append('fill betweens')
                self._to_setup.append(self.setupFillBetweens)
            if limit_to.count('images'):
                self.elements.append('images')
                self._to_setup.append(self.setupImages)
            if limit_to.count('bars'):
                self.elements.append('bars')
                self._to_setup.append(self.setupBars)

    def setupLines(self):
        """
        Enable and setup line plotting
        """

        # Enable line plotting
        self.plot = self.__plot
        self.updatePlotDataStyle = self.__updatePlotDataStyle
        self.updatePlotDataDelete = self.__updatePlotDataDelete

        # Initial  and insert table view for line plots
        self.tableViewLine = _QTableView()
        self.ui.modelTabWidget.addTab(self.tableViewLine, 'Lines')

        # Set model and delegates
        # Lines
        self.modelLine = _TableModelLines()
        self.delegateLine = _EditDelegateLines()
        self.tableViewLine.setModel(self.modelLine)
        self.tableViewLine.setItemDelegate(self.delegateLine)
        self.tableViewLine.show()


        # RESIZE COLUMNS
        header = self.tableViewLine.horizontalHeader()
        # alpha
        col = self.modelLine._COL_ALPHA
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewLine.setColumnWidth(col, new_width)

        # linewidth
        col = self.modelLine._COL_LINEWIDTH
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewLine.setColumnWidth(col, new_width)

        # markersize
        col = self.modelLine._COL_MARKERSIZE
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewLine.setColumnWidth(col, new_width)

        # delete
        col = self.modelLine._COL_DELETE
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewLine.setColumnWidth(col, new_width)

        # SIGNALS AND SLOTS

        # Make use of double-clicking within table
        self.tableViewLine.doubleClicked.connect(
            self.modelLine.doubleClickCheck)

        # When a model (table) elements changes or is deleted
        self.modelLine.dataChanged.connect(self.updatePlotDataStyle)
        self.modelLine.dataDeleted.connect(self.updatePlotDataDelete)

        # Export lines to csv
        self.ui.actionExport_Lines_to_CSV.setVisible(True)
        self.ui.actionExport_Lines_to_CSV.triggered.connect(self.export_lines_csv)

    def setupFillBetweens(self):
        """
        Enable and setup fill between plotting
        """

        # Enable fill_between plotting
        self.fill_between = self.__fill_between
        self.updateFillBetweenDataStyle = self.__updateFillBetweenDataStyle
        self.updateFillBetweenDataDelete = self.__updateFillBetweenDataDelete

        # Initial and insert table view for fill_between plots
        self.tableViewFillBetween = _QTableView()
        self.ui.modelTabWidget.addTab(self.tableViewFillBetween,
                                      'Fill Between')

        # Fill Between
        self.modelFillBetween = _TableModelFillBetween()
        self.delegateFillBetween = _EditDelegateFillBetween()
        self.tableViewFillBetween.setModel(self.modelFillBetween)
        self.tableViewFillBetween.setItemDelegate(self.delegateFillBetween)
        self.tableViewFillBetween.show()

        # RESIZE COLUMNS
        header = self.tableViewFillBetween.horizontalHeader()
        # alpha
        col = self.modelFillBetween._COL_ALPHA
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewFillBetween.setColumnWidth(col, new_width)

        # linewidth
        col = self.modelFillBetween._COL_LINEWIDTH
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewFillBetween.setColumnWidth(col, new_width)

        # delete
        col = self.modelFillBetween._COL_DELETE
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewFillBetween.setColumnWidth(col, new_width)

        # SIGNALS AND SLOTS

        # Make use of double-clicking within table
        self.tableViewFillBetween.doubleClicked.connect(
            self.modelFillBetween.doubleClickCheck)

        # When a model (table) elements changes or is deleted
        self.modelFillBetween.dataChanged.connect(self.updateFillBetweenDataStyle)
        self.modelFillBetween.dataDeleted.connect(self.updateFillBetweenDataDelete)

        # Export fillbetweens to csv
        self.ui.actionExport_Fill_Between_to_CSV.setVisible(True)
        self.ui.actionExport_Fill_Between_to_CSV.triggered.connect(self.export_fillbetweens_csv)

    def setupImages(self):
        """
        Enable and setup image plotting
        """

        # Enable imaging
        self.imshow = self.__imshow
        self.updateImagesDataStyle = self.__updateImagesDataStyle
        self.updateImagesDataDelete = self.__updateImagesDataDelete

        # images data-- similar to plot_data above

        # Initial  and insert table view for images
        self.tableViewImages = _QTableView()
        self.ui.modelTabWidget.addTab(self.tableViewImages, 'Images')

        # Images
        self.modelImages = _TableModelImages()
        self.delegateImages = _EditDelegateImages()
        self.tableViewImages.setModel(self.modelImages)
        self.tableViewImages.setItemDelegate(self.delegateImages)
        self.tableViewImages.show()

        # RESIZE COLUMNS
        header = self.tableViewImages.horizontalHeader()
        # alpha
        col = self.modelImages._COL_ALPHA
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewImages.setColumnWidth(col, new_width)

        # clim low
        col = self.modelImages._COL_CLIM_LOW
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewImages.setColumnWidth(col, new_width)

        # clim high
        col = self.modelImages._COL_CLIM_HIGH
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewImages.setColumnWidth(col, new_width)

        # delete
        col = self.modelImages._COL_DELETE
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewImages.setColumnWidth(col, new_width)

        # SIGNALS AND SLOTS

        # Make use of double-clicking within table
        self.tableViewImages.doubleClicked.connect(
            self.modelImages.doubleClickCheck)

        # When a model (table) elements changes or is deleted
        self.modelImages.dataChanged.connect(self.updateImagesDataStyle)
        self.modelImages.dataDeleted.connect(self.updateImagesDataDelete)

    def setupBars(self):
        """
        Enable and setup bar and histogram plotting
        """

        # Enable bar plotting
        self.bar = self.__bar
        self.hist = self.__hist
        self.updateBarsDataStyle = self.__updateBarsDataStyle
        self.updateBarsDataDelete = self.__updateBarsDataDelete

        # Initial  and insert table view for bars
        self.tableViewBars = _QTableView()
        self.ui.modelTabWidget.addTab(self.tableViewBars, 'Bars')


        # Bars/Bars
        self.modelBars = _TableModelBars()
        self.delegateBars = _EditDelegateBars()
        self.tableViewBars.setModel(self.modelBars)
        self.tableViewBars.setItemDelegate(self.delegateBars)
        self.tableViewBars.show()

        # RESIZE COLUMNS
        header = self.tableViewBars.horizontalHeader()
        # alpha
        col = self.modelBars._COL_ALPHA
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewBars.setColumnWidth(col, new_width)

        # linewidth
        col = self.modelBars._COL_LINEWIDTH
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewBars.setColumnWidth(col, new_width)

        # widthfactor
        col = self.modelBars._COL_WIDTH_FACTOR
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewBars.setColumnWidth(col, new_width)

        # delete
        col = self.modelBars._COL_DELETE
        new_width = int(1.1*header.sectionSizeHint(col))
        self.tableViewBars.setColumnWidth(col, new_width)

        # SIGNALS AND SLOTS

        # Make use of double-clicking within table
        self.tableViewBars.doubleClicked.connect(
            self.modelBars.doubleClickCheck)

        # When a model (table) elements changes or is deleted
        self.modelBars.dataChanged.connect(self.updateBarsDataStyle)
        self.modelBars.dataDeleted.connect(self.updateBarsDataDelete)

        # Export bars to csv
        self.ui.actionExport_Bars_to_CSV.setVisible(True)
        self.ui.actionExport_Bars_to_CSV.triggered.connect(self.export_bars_csv)

    def setup(self, limit_to=None, parent=None):
        """
        Basic UI setup
        """

        # Generic start to any pyQT program
        
        
        super(SciPlotUI, self).__init__(parent)
        self.ui = Ui_Plotter()
        self.ui.setupUi(self)
        self.setSizePolicy(_QSizePolicy.Expanding,
                           _QSizePolicy.Expanding)

        # Global "data" i.e., title, x-label, y-label, etc
        self._global_data = _DataGlobal()

        # MPL plot widget
        self.mpl_widget = _MplCanvas(height=6, dpi=100)

        # Hold is deprecated in MPL2
        if self._mpl_v1:
            self.mpl_widget.ax.hold(True)

        # Insert MPL widget and toolbar
        self.ui.verticalLayout.insertWidget(0, self.mpl_widget, 0, _QtCore.Qt.AlignHCenter)
        
        self.ui.verticalLayout.insertWidget(0, self.mpl_widget.toolbar,0, _QtCore.Qt.AlignHCenter)
        self.updateAxisParameters()
        self.updateFigureParameters()
        self.mpl_widget.draw()

        # Insert TabWidget
        self.ui.modelTabWidget = _QTabWidget()
        self.ui.verticalLayout.insertWidget(-1, self.ui.modelTabWidget)

        # Setup what tabs are available:
        self._tabAvailability(limit_to)
        for count in self._to_setup:
            count()

        # SIGNALS AND SLOTS
        
        # Global labels
        self.ui.lineEditTitle.editingFinished.connect(self.updateLabelsFromLineEdit)
        self.ui.lineEditXLabel.editingFinished.connect(self.updateLabelsFromLineEdit)
        self.ui.lineEditYLabel.editingFinished.connect(self.updateLabelsFromLineEdit)

        # Non-tracked (not saved) properties
        self.ui.comboBoxAspect.currentIndexChanged.connect(self.axisAspect)
        self.ui.comboBoxAxisScaling.currentIndexChanged.connect(self.axisScaling)
        self.ui.checkBoxAxisVisible.stateChanged.connect(self.axisVisible)
        self.ui.lineEditXLimMin.editingFinished.connect(self.axisLimits)
        self.ui.lineEditXLimMax.editingFinished.connect(self.axisLimits)
        self.ui.lineEditYLimMin.editingFinished.connect(self.axisLimits)
        self.ui.lineEditYLimMax.editingFinished.connect(self.axisLimits)
        
        # Actions
        self.ui.pushButtonClearAll.pressed.connect(self.clearAll)
        self.ui.pushButtonDefaultView.pressed.connect(self.defaultView)

        # Formatting Features
        self.ui.actionFigureDPI.triggered.connect(self.figureDPI)
        self.ui.actionFigureSavedDPI.triggered.connect(self.figureSaveDPI)
        self.ui.actionFigure_Size_Display.triggered.connect(self.figureSizeDisplay)
        self.ui.pushButtonApplyFigParams.pressed.connect(self.applyFigProps)

    def __plot(self, x, y, label=None, x_label=None, y_label=None, meta={}, 
               **kwargs):
        """
        MPL-like plotting functionality

        Parameters
        ----------
        x : ndarray (1D)
            X-axis data

        y : ndarray (1D, for now)
            Y-axis data

        label : str
            Label of plot

        x_label : str
            X-axis label (units)

        y_label : str
            Y-axis label (units)

        kwargs : dict
            Other parameters sent directly to mpl-plot

        """

        # Temporary plot-data
        plot_data = _DataLine()
        plot_data.x = x
        plot_data.y = y
        plot_data.label = label
        plot_data.meta = meta
        plot_data.id = _time.time()

        # Plot outputs a line object
        plot_data.mplobj = self.mpl_widget.ax.plot(x, y, label=label, **kwargs)
        try:
            self.mpl_widget.ax.legend(loc='best')
        except:
            pass

        # If labels are provided, update the global data and the linEdits
        if x_label is not None or y_label is not None:
            self.updateAllLabels(x_label=x_label, y_label=y_label)

        self.mpl_widget.fig.tight_layout()
        self.axisAspect()
        self.mpl_widget.draw()


        # Since the plot was not fed style-info (unless kwargs were used)
        # we rely on the mpl stylesheet to setup color, linewidth, etc.
        # Thus, we plot, then retrieve what the style info was
        plot_data.retrieve_style_from_line(plot_data.mplobj[0])

        # Append this specific plot data to out list of all plots
        self.list_ids.append(plot_data.id)
        self.list_all.append(plot_data)

        # Update model
        self.modelLine._model_data.append(plot_data.model_style)
        self.modelLine.layoutChanged.emit()

    def updateMplLabels(self, x_label=None, y_label=None, title=None):
        """
        Within the MPL widget, update the x- and y-labels and the title
        """
        if x_label is not None:
            self.mpl_widget.ax.set_xlabel(x_label)

        if y_label is not None:
            self.mpl_widget.ax.set_ylabel(y_label)

        if title is not None:
            self.mpl_widget.ax.set_title(title)
        self.mpl_widget.fig.tight_layout()
        self.mpl_widget.draw()

    def updateDataLabels(self, x_label=None, y_label=None, title=None):
        """
        Within the global data container, update the x- and y-labels and the \
        title
        """
        if x_label is not None:
            self._global_data.labels['x_label'] = x_label

        if y_label is not None:
            self._global_data.labels['y_label'] = y_label

        if title is not None:
            self._global_data.labels['title'] = title

    def updateLineEditLabels(self, x_label=None, y_label=None, title=None):
        """
        Within the pyQT lineEdit widgets, update the x- and y-labels and the \
        title
        """
        if x_label is not None:
            self.ui.lineEditXLabel.setText(x_label)

        if y_label is not None:
            self.ui.lineEditYLabel.setText(y_label)

        if title is not None:
            self.ui.lineEditTitle.setText(title)

    def updateAllLabels(self, x_label=None, y_label=None, title=None):
        """
        Update the x- and y-labels and the title in the MPL widget, the \
        lineEdit boxes, and the global data container
        """
        self.updateMplLabels(x_label=x_label, y_label=y_label, title=title)
        self.updateDataLabels(x_label=x_label, y_label=y_label, title=title)
        self.updateLineEditLabels(x_label=x_label, y_label=y_label,
                                  title=title)

    def updateLabelsFromLineEdit(self):
        """
        From the linEdit widgets, update the x- and y-labels and the title \
        in the MPL widget and the global data container
        """
        title = None
        x_label = None
        y_label = None

        sender = self.sender()
        if sender == self.ui.lineEditTitle:
            title = self.ui.lineEditTitle.text()
        elif sender == self.ui.lineEditXLabel:
            x_label = self.ui.lineEditXLabel.text()
        elif sender == self.ui.lineEditYLabel:
            y_label = self.ui.lineEditYLabel.text()
        self.updateDataLabels(x_label=x_label, y_label=y_label, title=title)
        self.updateMplLabels(x_label=x_label, y_label=y_label, title=title)

    def __updatePlotDataStyle(self):
        """
        Something style-related changed in the model; thus, need to change \
        these elements in the plot data
        """
        for num, style_info in enumerate(self.modelLine._model_data):
            idx = self.list_ids.index(style_info['id'])
            self.list_all[idx].model_style = style_info
        self.refreshAllPlots()

    def __updatePlotDataDelete(self, row, plt_id):
        """
        A plot was deleted (likely from within the model); thus, need to \
        remove the corresponding plot data
        """
        try:
            idx_to_remove = self.list_ids.index(plt_id)
            self.list_ids.pop(idx_to_remove)
            self.list_all.pop(idx_to_remove)
        except:
            print('Error in __updatePlotDataDelete: {}'.format(idx_to_remove))
            
        self.refreshAllPlots()

    def refreshAllPlots(self):
        """
        Clear and re-plot all plot data of all types
        """

        # Clear axis -- in the future, maybe clear figure and recreate axis
        self.mpl_widget.ax.clear()
        
        for itm in self.list_all:
            if isinstance(itm, _DataLine):
                if self._mpl_v1:
                    self.mpl_widget.ax.hold(True)
                
                # Hide label if alpha=0
                if itm.style_dict['alpha'] == 0:
                    label = None
                else:
                    label = itm.label
                itm.mplobj = self.mpl_widget.ax.plot(itm.x, itm.y, 
                                                     label=label,
                                                     color=itm.style_dict['color'],
                                                     alpha=itm.style_dict['alpha'],
                                                     linewidth=itm.style_dict['linewidth'],
                                                     linestyle=itm.style_dict['linestyle'],
                                                     marker=itm.style_dict['marker'],
                                                     markersize=itm.style_dict['markersize'])
            elif isinstance(itm, _DataBar):
#                print('Bar')
                if self._mpl_v1:
                    self.mpl_widget.ax.hold(True)
                
                # Hide label if alpha=0
                if itm.style_dict['alpha'] == 0:
                    label = None
                else:
                    label = itm.label
                itm.mplobj = self.mpl_widget.ax.bar(itm._left, itm.y, 
                                                    bottom=itm.bottom,
                                                    width=itm._width,
                                                    label=label,
                                                    facecolor=itm.style_dict['facecolor'],
                                                    alpha=itm.style_dict['alpha'],
                                                    edgecolor=itm.style_dict['edgecolor'],
                                                    linewidth=itm.style_dict['linewidth'])
            elif isinstance(itm, _DataImages):
#                print('Images')
                if self._mpl_v1:
                    self.mpl_widget.ax.hold(True)
                
                # Hide label if alpha=0
                if itm.style_dict['alpha'] == 0:
                    label = None
                else:
                    label = itm.label
                if itm.cbar['obj'] is not None:
                    try:  # Have had some unknown exceptions with .remove()
                        itm.cbar['obj'].remove()
                        itm.cbar['obj'] = None
                    except:
                        pass
                itm.mplobj = self.mpl_widget.ax.imshow(itm.img, label=label,
                                                       interpolation='none',
                                                       origin='lower',
                                                       cmap=_mpl.cm.cmap_d[itm.style_dict['cmap_name']],
                                                       alpha=itm.style_dict['alpha'],
                                                       clim=itm.style_dict['clim'])
                if itm.cbar['show']:
                    itm.cbar['obj'] = self.mpl_widget.fig.colorbar(itm.mplobj,
                                                                   use_gridspec=True)
            elif isinstance(itm, _DataFillBetween):
#                print('Fill Between')
                if self._mpl_v1:
                    self.mpl_widget.ax.hold(True)
                
                # Hide label if alpha=0
                if itm.style_dict['alpha'] == 0:
                    label = None
                else:
                    label = itm.label
                itm.mplobj = self.mpl_widget.ax.fill_between(itm.x, itm.y_low, itm.y_high,
                                                  label=label,
                                                  facecolor=itm.style_dict['facecolor'],
                                                  edgecolor=itm.style_dict['edgecolor'],
                                                  alpha=itm.style_dict['alpha'],
                                                  linewidth=itm.style_dict['linewidth'])

            else:
                print('Unknown')

        # Only add legend if legend handles exist
        h,l = self.mpl_widget.ax.get_legend_handles_labels()
        if l:
            self.mpl_widget.ax.legend(loc='best')

        # Apply x- and y-labels and a title if they are set
        if self._global_data.labels['title'] is not None:
            self.mpl_widget.ax.set_title(self._global_data.labels['title'])
        if self._global_data.labels['x_label'] is not None:
            self.mpl_widget.ax.set_xlabel(self._global_data.labels['x_label'])
        if self._global_data.labels['y_label'] is not None:
            self.mpl_widget.ax.set_ylabel(self._global_data.labels['y_label'])

        self.mpl_widget.fig.tight_layout()
        self.updateAxisParameters()
        self.updateFigureParameters()
        self.axisAspect()
        self.mpl_widget.draw()

    def __fill_between(self, x, y_low, y_high, label=None,  meta={},
                       x_label=None, y_label=None, **kwargs):
        """
        MPL-like fill_between plotting functionality

        Parameters
        ----------
        x : ndarray (1D)
            X-axis data

        y_low : ndarray (1D, for now)
            Low Y-axis data

        y_high : ndarray (1D, for now)
            High Y-axis data

        label : str
            Label of plot

        x_label : str
            X-axis label (units)

        y_label : str
            Y-axis label (units)

        kwargs : dict
            Other parameters sent directly to mpl-fill_between

        """

        # Temporary fill_between-data
        fill_between_data = _DataFillBetween()
        fill_between_data.x = x
        fill_between_data.y_low = y_low
        fill_between_data.y_high = y_high
        fill_between_data.label = label
        fill_between_data.meta = meta
        fill_between_data.id = _time.time()

        # Fill between outputs a polycollection
        fill_between_data.mplobj = self.mpl_widget.ax.fill_between(x, y_low, y_high,
                                                   label=label, **kwargs)
        self.mpl_widget.ax.legend(loc='best')
        self.mpl_widget.fig.tight_layout()
        self.axisAspect()
        self.mpl_widget.draw()

        # Since the fill_between was not fed style-info (unless kwargs were used)
        # we rely on the mpl stylesheet to setup color, linewidth, etc.
        # Thus, we plot, then retrieve what the style info was
        fill_between_data.retrieve_style_from_polycollection(fill_between_data.mplobj)

        # Append this specific plot data to out list of all plots
        self.list_ids.append(fill_between_data.id)
        self.list_all.append(fill_between_data)

        # Update model
        self.modelFillBetween._model_data.append(fill_between_data.model_style)
        self.modelFillBetween.layoutChanged.emit()
        

    def __updateFillBetweenDataStyle(self):
        """
        Something style-related changed in the model; thus, need to change \
        these elements in the fill_between data
        """
        for num, style_info in enumerate(self.modelFillBetween._model_data):
            idx = self.list_ids.index(style_info['id'])
            self.list_all[idx].model_style = style_info
        self.refreshAllPlots()

    def __updateFillBetweenDataDelete(self, row, plt_id):
        """
        A plot was deleted (likely from within the model); thus, need to \
        remove the corresponding plot data
        """
        idx_to_remove = self.list_ids.index(plt_id)
        self.list_ids.pop(idx_to_remove)
        self.list_all.pop(idx_to_remove)
        
        self.refreshAllPlots()

    def __imshow(self, img, x=None, y=None, label=None, meta={},
               x_label=None, y_label=None, cbar=False, **kwargs):
        """
        MPL-like plotting functionality

        Parameters
        ----------
        img : ndarray (2D)
            Image data

        x : ndarray (1D)
            X-axis data

        y : ndarray (1D, for now)
            Y-axis data

        label : str
            Label of plot

        x_label : str
            X-axis label (units)

        y_label : str
            Y-axis label (units)
            
        cbar : bool
            Attach a colorbar to the img

        kwargs : dict
            Other parameters sent directly to mpl-imshow

        """

        # Temporary plot-data
        image_data = _DataImages()
        image_data.img = img
        image_data.x = x
        image_data.y = y
        image_data.label = label
        image_data.meta = meta
        image_data.id = _time.time()
        image_data.cbar['show'] = cbar

        # Imshow outputs an image object
        image_data.mplobj = self.mpl_widget.ax.imshow(img, interpolation='None',
                                                origin='lower',
                                                label=label,
                                                **kwargs)
        if image_data.cbar['show']:
            image_data.cbar['obj'] = self.mpl_widget.fig.colorbar(image_data.mplobj,
                                                use_gridspec=True)
            
#        self.mpl_widget.ax.legend(loc='best')

        # If labels are provided, update the global data and the linEdits
        if x_label is not None or y_label is not None:
            self.updateAllLabels(x_label=x_label, y_label=y_label)

        self.mpl_widget.fig.tight_layout()
        
        self.axisAspect()
        self.mpl_widget.draw()

        # Since the image was not fed style-info (unless kwargs were used)
        # we rely on the mpl stylesheet to setup cmap, etc.
        # Thus, we plot, then retrieve what the style info was
        image_data.retrieve_style_from_image(image_data.mplobj)

        # Append this specific plot data to out list of all plots
        self.list_ids.append(image_data.id)
        self.list_all.append(image_data)

        # Update model
        self.modelImages._model_data.append(image_data.model_style)
        self.modelImages.layoutChanged.emit()


    def __updateImagesDataStyle(self):
        """
        Something style-related changed in the model; thus, need to change \
        these elements in the fill_between data
        """
        for num, style_info in enumerate(self.modelImages._model_data):
            idx = self.list_ids.index(style_info['id'])
            self.list_all[idx].model_style = style_info
        self.refreshAllPlots()

    def __updateImagesDataDelete(self, row, plt_id):
        """
        A plot was deleted (likely from within the model); thus, need to \
        remove the corresponding plot data
        """
        
        idx_to_remove = self.list_ids.index(plt_id)
        self.list_ids.pop(idx_to_remove)
        popd = self.list_all.pop(idx_to_remove)
            
        if popd.cbar['obj'] is not None:
            popd.cbar['obj'].remove()
#        self.axisAspect()
        self.refreshAllPlots()


    def __bar(self, x, y, bottom=0, width_factor=1.0, use_real_width=False, 
              label=None, meta={}, x_label=None, y_label=None, **kwargs):
        """
        MPL-like plotting functionality

        Note
        ----
        Unlike MPL bar, this method uses centered data. Thus, x is the center \
        position of the bar

        Parameters
        ----------
        x : ndarray (1D)
            X-axis data (center of bars)

        y : ndarray (1D, for now)
            Y-axis data (height)

        bottom : float (for now)
            Baseline of bars
            
        width_factor: float
            If legnth of y>1, fraction of space between bars taken up by bar \
            (e.g. 1.0 leads to bars that tough). If y is a single-value OR \
            use_real_width is True), is the width of the bar.

        use_real_width : bool, optional (default=False):
            If True, width_factor is the real width (in x-units)
            
        label : str
            Label of plot

        x_label : str
            X-axis label (units)

        y_label : str
            Y-axis label (units)

        kwargs : dict
            Other parameters sent directly to mpl-plot

        """

        # Temporary plot-data
        bar_data = _DataBar()
        bar_data.x = x
        bar_data.y = y
        bar_data.bottom = bottom
        bar_data.label = label
        bar_data.meta = meta
        bar_data.id = _time.time()

        bar_data.style_dict['width_factor'] = width_factor

        _multi_value = None

        if isinstance(y, (int, float)):
            _multi_value = False
        if isinstance(y, _np.ndarray):
            if y.size == 1:
                _multi_value = False
            else:
                _multi_value = True
        if isinstance(y, (list, tuple)):
            if len(y) == 1:
                _multi_value = False
            else:
                _multi_value = True

        if _multi_value and use_real_width == False:
            # Distance between bars
            bar_data._gap = _np.abs(x[1]-x[0])

            # Width of a bar is a fraction of the gap
            bar_data._width = bar_data._gap*bar_data.style_dict['width_factor']

        else:
            # Single-valued: no gap
            bar_data._gap = None
            bar_data._width = width_factor

        # MPL-bar uses left-edge rather than center
        bar_data._left = bar_data.x - bar_data._width/2

        # Plot outputs a list of patch objects
        bar_data.mplobj = self.mpl_widget.ax.bar(bar_data._left, y,
                                           bottom=bar_data.bottom,
                                           width=bar_data._width,
                                           label=label, **kwargs)
        self.mpl_widget.ax.legend(loc='best')

        # If labels are provided, update the global data and the linEdits
        if x_label is not None or y_label is not None:
            self.updateAllLabels(x_label=x_label, y_label=y_label)

        self.mpl_widget.fig.tight_layout()
        
        self.axisAspect()
        self.mpl_widget.draw()


        # Since the plot was not fed style-info (unless kwargs were used)
        # we rely on the mpl stylesheet to setup color, linewidth, etc.
        # Thus, we plot, then retrieve what the style info was
        bar_data.retrieve_style_from_bar(bar_data.mplobj[0])
    
        # Append this specific plot data to out list of all plots
        self.list_ids.append(bar_data.id)
        self.list_all.append(bar_data)
        
        # Update model
        self.modelBars._model_data.append(bar_data.model_style)
        self.modelBars.layoutChanged.emit()
        
        # Note: New in MPL2, edgecolor is RGBA with A defaulting to 0
        # (ie transparent, which Sciplot does not currently support).
        self.refreshAllPlots()

    def __hist(self, data, bins=10, label=None, meta={}, x_label=None,
             y_label='Counts', **kwargs):
        """
        MPL-like histogram plotting

        Parameters
        ----------
        data : ndarray (1D, for now)
            Data (center of bars)

        bins : int
            Number of histogram bins

        label : str
            Label of plot

        x_label : str
            X-axis label (units)

        y_label : str
            Y-axis label (units)

        kwargs : dict
            Other parameters sent directly to mpl-plot

        """
        counts, lefts = _np.histogram(data, bins=bins)
        gap = _np.abs(lefts[1] - lefts[0])
        offset = gap/2

        self.bar(lefts[:-1]+offset, counts, width_factor=1.0, label=label,
                 x_label=x_label, y_label=y_label, meta=meta, **kwargs)

    def __updateBarsDataStyle(self):
        """
        Something style-related changed in the model; thus, need to change \
        these elements in the fill_between data
        """
        for num, style_info in enumerate(self.modelBars._model_data):
            idx = self.list_ids.index(style_info['id'])
            self.list_all[idx].model_style = style_info
        self.refreshAllPlots()

    def __updateBarsDataDelete(self, row, plt_id):
        """
        A plot was deleted (likely from within the model); thus, need to \
        remove the corresponding plot data
        """
        idx_to_remove = self.list_ids.index(plt_id)
        self.list_ids.pop(idx_to_remove)
        self.list_all.pop(idx_to_remove)
        
        self.refreshAllPlots()

    def axisAspect(self):
        """
        Set axis aspect ratio property
        """
        aspect = self.ui.comboBoxAspect.currentText()
        self.mpl_widget.ax.set_aspect(aspect)
        self.mpl_widget.fig.tight_layout()
        self.updateAxisParameters()
        self.updateFigureParameters()
        self.mpl_widget.draw()

    def axisScaling(self):
        """
        Set axis scaling property
        """
        ratio = self.ui.comboBoxAxisScaling.currentText()
        self.mpl_widget.ax.axis(ratio)
        self.mpl_widget.fig.tight_layout()
        self.updateAxisParameters()
        self.updateFigureParameters()
        self.mpl_widget.draw()

    def axisVisible(self):
        """
        Set whether axis is on or off
        """
        state = self.ui.checkBoxAxisVisible.isChecked()
        if state:
            state = 'on'
        else:
            state = 'off'

        self.mpl_widget.ax.axis(state)
        self.mpl_widget.fig.tight_layout()
        self.updateAxisParameters()
        self.updateFigureParameters()
        self.mpl_widget.draw()

    def axisLimits(self):
        """
        Set axis limits
        """
        if self.sender() == self.ui.lineEditXLimMin:
            value = float(self.ui.lineEditXLimMin.text())
            self.mpl_widget.ax.axis(xmin=value)
        elif self.sender() == self.ui.lineEditXLimMax:
            value = float(self.ui.lineEditXLimMax.text())
            self.mpl_widget.ax.axis(xmax=value)
        elif self.sender() == self.ui.lineEditYLimMin:
            value = float(self.ui.lineEditYLimMin.text())
            self.mpl_widget.ax.axis(ymin=value)
        elif self.sender() == self.ui.lineEditYLimMax:
            value = float(self.ui.lineEditYLimMax.text())
            self.mpl_widget.ax.axis(ymax=value)

        self.mpl_widget.fig.tight_layout()
        self.updateAxisParameters()
        self.updateFigureParameters()
        self.mpl_widget.draw()

    def updateAxisParameters(self):
        """
        Query current state of axis settings and update appropriate lineEdit's
        """
        axis_visible = self.mpl_widget.ax.axison
        self.ui.checkBoxAxisVisible.setChecked(axis_visible)
        xmin, xmax, ymin, ymax = self.mpl_widget.ax.axis()
        self.ui.lineEditXLimMin.setText(str(round(xmin,5)))
        self.ui.lineEditXLimMax.setText(str(round(xmax,5)))
        self.ui.lineEditYLimMin.setText(str(round(ymin,5)))
        self.ui.lineEditYLimMax.setText(str(round(ymax,5)))

    def updateFigureParameters(self):
        """
        Query current state of axis settings and update appropriate lineEdit's
        """
        fig_dpi = self.mpl_widget.fig.get_dpi()
        save_dpi = _mpl.rcParams['savefig.dpi']
        if save_dpi == 'figure':
            save_dpi = fig_dpi
        fig_size = self.mpl_widget.fig.get_size_inches()
        
        self.ui.spinBoxFigureDPI.setValue(fig_dpi)
        self.ui.spinBoxFigureSavedDPI.setValue(save_dpi)
        self.ui.spinBoxFigSizeWidth.setValue(fig_size[0])
        self.ui.spinBoxFigSizeHeight.setValue(fig_size[1])
        
    def defaultView(self):
        """
        Set default and Home view to the current one
        """
        # New versions of MPL don't have these functions
        try:
            self.mpl_widget.toolbar._views.clear()
            self.mpl_widget.toolbar._positions.clear()
        except:
            pass

        self.mpl_widget.toolbar.update()
    
    def clearAllBars(self):
        try:
            self.modelBars._model_data = []
            ids = self.list_bar_ids
            for i in ids:
                self.clearID(i)
            
            self.modelBars.layoutChanged.emit()
        except:
            print('Error in clearAllBars')

    def clearID(self, clear_id):
        idx_to_remove = self.list_ids.index(clear_id)
        self.list_ids.pop(idx_to_remove)
        self.list_all.pop(idx_to_remove)

    def clearAll(self):
        """
        Clear all plots and graphs and images
        """
        try:
            self.modelLine._model_data = []
            self.modelLine.layoutChanged.emit()
        except:
            print('Error in clear all of plots/lines')
        
        try:
            self.modelBars._model_data = []
            self.modelBars.layoutChanged.emit()
        except:
            print('Error in clear all of bars')
        
        
        try:
            # Need to iterate as to check for colorbar existance
            for num, model_data in enumerate(self.modelImages._model_data):
                idx_to_remove = self.list_ids.index(model_data['id'])
                self.list_ids.pop(idx_to_remove)
                popd = self.list_all.pop(idx_to_remove)
                if popd.cbar['obj'] is not None:
                    popd.cbar['obj'].remove()
            
            self.modelImages._model_data = []
            self.modelImages.layoutChanged.emit()
        except:
            print('Error in clear all of images')
        
        try:
            self.modelFillBetween._model_data = []
            self.modelFillBetween.layoutChanged.emit()
        except:
            print('Error in clear all of fill-betweens')
            
        try:
            self.list_ids = []
            self.list_all = []
        except:
            print('Error in clear all')
        finally:
            self.refreshAllPlots()
            self.all_cleared.emit(id(self))

    def export_bars_csv(self):
        ret = _QFileDialog.getSaveFileName(filter="Comma-Separated Values (*.csv);;All Files (*.*)")
        if ret[0]:
            # pth, fname = _os.path.split(ret[0])
            with open(ret[0],'w') as f:
                for q in self.list_bar_objs:
                    f.write('{}\n'.format(q.label))
                    f.write('left,')
                    q._left.tofile(f, sep=',')
                    f.write('\nx,')
                    q.x.tofile(f, sep=',')
                    f.write('\ny,')
                    q.y.tofile(f,sep=',')
                    f.write('\n\n')    

    def export_lines_csv(self):
        ret = _QFileDialog.getSaveFileName(filter="Comma-Separated Values (*.csv);;All Files (*.*)")
        if ret[0]:
            # pth, fname = _os.path.split(ret[0])
            with open(ret[0],'w') as f:
                for q in self.list_line_objs:
                    f.write('{}\n'.format(q.label))
                    f.write('x,')
                    q.x.tofile(f, sep=',')
                    f.write('\ny,')
                    q.y.tofile(f,sep=',')
                    f.write('\n\n')

    def export_fillbetweens_csv(self):
        ret = _QFileDialog.getSaveFileName(filter="Comma-Separated Values (*.csv);;All Files (*.*)")
        if ret[0]:
            # pth, fname = _os.path.split(ret[0])
            with open(ret[0],'w') as f:
                for q in self.list_fillbetween_objs:
                    f.write('{}\n'.format(q.label))
                    f.write('x,')
                    q.x.tofile(f, sep=',')
                    f.write('\ny_low,')
                    q.y_low.tofile(f,sep=',')
                    f.write('\ny_high,')
                    q.y_high.tofile(f,sep=',')
                    f.write('\n\n')

    @property
    def n_lines(self):
        return sum(isinstance(x, _DataLine) for x in self.list_all)
    
    @property
    def n_bars(self):
        return sum(isinstance(x, _DataBar) for x in self.list_all)
        
    @property
    def n_fillbetweens(self):
        return sum(isinstance(x, _DataFillBetween) for x in self.list_all)
        
    @property
    def n_images(self):
        return sum(isinstance(x, _DataImages) for x in self.list_all)
        
    @property
    def list_line_objs(self):
        return [x for x in self.list_all if isinstance(x, _DataLine)]
        
    @property
    def list_line_ids(self):
        return [x.id for x in self.list_all if isinstance(x, _DataLine)]
        
    @property
    def list_bar_objs(self):
        return [x for x in self.list_all if isinstance(x, _DataBar)]
        
    @property
    def list_bar_ids(self):
        return [x.id for x in self.list_all if isinstance(x, _DataBar)]
        
    @property
    def list_fillbetween_objs(self):
        return [x for x in self.list_all if isinstance(x, _DataFillBetween)]
        
    @property
    def list_fillbetween_ids(self):
        return [x.id for x in self.list_all if isinstance(x, _DataFillBetween)]
        
    @property
    def list_image_objs(self):
        return [x for x in self.list_all if isinstance(x, _DataImages)]
        
    @property
    def list_image_ids(self):
        return [x.id for x in self.list_all if isinstance(x, _DataImages)]
        
    def figureDPI(self):
        curr_dpi = self.mpl_widget.fig.get_dpi()
        dpi, okPressed = _QInputDialog.getInt(self, "New Figure DPI","DPI:", curr_dpi, 10, 100000, 25)
        if okPressed:
            self.mpl_widget.fig.set_dpi(dpi)
            self.mpl_widget.updateGeometry()
            self.mpl_widget.draw()
        self.updateFigureParameters()

    def figureSaveDPI(self):
        curr_dpi = _mpl.rcParams['savefig.dpi']
        if not isinstance(curr_dpi, int):
            # Savefig is set to 'figure'
            curr_dpi = self.mpl_widget.fig.get_dpi()
        
        dpi, okPressed = _QInputDialog.getInt(self, "New DPI for Saved Figures","DPI:", curr_dpi, 10, 100000, 25)
        if okPressed:
            _mpl.rcParams['savefig.dpi'] = dpi
        self.updateFigureParameters()

    def figureSizeDisplay(self):
        curr_size = self.mpl_widget.fig.get_size_inches()
        new_size, okPressed = DualEntry.getDualEntries(curr_size[0], curr_size[1], input_type=float, text="Figure Size (W x H inches)", parent=self)
        if okPressed:
            self.mpl_widget.updateGeometry()
            self.mpl_widget.fig.set_size_inches(new_size[0], new_size[1], forward=True)
            self.mpl_widget.draw()
        self.updateFigureParameters()

    def applyFigProps(self):
        """ Apply manually-entered figure properties (e.g., dpi) """
        
        new_fig_size = (self.ui.spinBoxFigSizeWidth.value(), self.ui.spinBoxFigSizeHeight.value())
        new_fig_dpi = self.ui.spinBoxFigureDPI.value()
        new_save_dpi = self.ui.spinBoxFigureSavedDPI.value()

        self.mpl_widget.fig.set_dpi(new_fig_dpi)
        self.mpl_widget.fig.set_size_inches(new_fig_size)
        _mpl.rcParams['savefig.dpi'] = new_save_dpi
        self.mpl_widget.updateGeometry()
        self.mpl_widget.draw()

if __name__ == '__main__':

    app = _QApplication(_sys.argv)
    #app.setQuitOnLastWindowClosed(True)
    
    winPlotter = SciPlotUI(limit_to=['lines','bars', 'fill betweens',
                                     'images'])
    #winPlotter.show()

    x = _np.arange(100)
    y = x**2

    winPlotter.plot(x, y, x_label='X', label='Plot')
#    winPlotter.plot(x, y**1.1, label='Plot 2')
    winPlotter.fill_between(x, y-1000, y+1000, label='Fill Between')
#
    winPlotter.imshow(_np.random.randn(100,100), label='Imshow', cbar=True)
    winPlotter.bar(x[::10],y[::10],label='Bar')
#    winPlotter.hist(y,label='Hist')

#    winPlotter.bar(0,10, label='Bar: single-value')
    app.exec_()



# -*- coding: utf-8 -*-
"""
Created on Thu Jul  7 15:06:04 2016

@author: chc
"""

import matplotlib as _mpl


class PolyCollectionStyle:
    """
    Style information for each data plotted (polycollection)
    """
    def __init__(self):
        self._setupPolyCollectionStyle()

    def _setupPolyCollectionStyle(self):
        self.style_dict = {'facecolor': None,
                           'alpha': None,
                           'edgecolor': None,
                           'linewidth': None}

    def retrieve_style_from_polycollection(self, pc):
        """
        Take an MPL polycollection object and retrieve appropriate attributes
        """

        # facecolor p-collections return arrays of RGBA
        # Not going to worry about different face colors yet
        color = pc.get_facecolors()[0][:-1]
        if isinstance(color, str):
            # color = _mpl.colors.ColorConverter.cache[color]
            color = _mpl.colors.ColorConverter().to_rgb(color)
        if isinstance(color, tuple):
            color = list(color)
        self.style_dict['facecolor'] = color

        # Alpha (transparency)
        alpha = pc.get_alpha()
        if alpha is None:
            alpha = 1
        self.style_dict['alpha'] = alpha

        # edgecolor p-collections return arrays of RGBA
        # Not going to worry about different edge colors yet
        if pc.get_edgecolors().size > 0:  # MPL 2.0 starts with no edgecolors
            color = pc.get_edgecolors()[0][:-1]
        else:
            pass
        if isinstance(color, str):
            # color = _mpl.colors.ColorConverter.cache[color]
            color = _mpl.colors.ColorConverter().to_rgb(color)
        if isinstance(color, tuple):
            color = list(color)
        self.style_dict['edgecolor'] = color

        # Linewidth (p-collection return tuples of len 1)
        self.style_dict['linewidth'] = pc.get_linewidth()[0]


# Not actually going to use polycollection raw data
# So I'm not going to create this
#
#class DataPolyCollection(_Data, PolyCollectionStyle):
#    def __init__(self):
#        self._setupData()
#        self._setupLineStyle()
#
#    @property
#    def model_style(self):
#        out = {}
#        out.update(self.style_dict)
#        out['label'] = self.label
#        return out
#
#    @model_style.setter
#    def model_style(self, value):
#        self.label = value['label']
#        self.style_dict['color'] = value['color']
#        self.style_dict['alpha'] = value['alpha']
#        self.style_dict['linewidth'] = value['linewidth']
#        self.style_dict['linestyle'] = value['linestyle']
#        self.style_dict['marker'] = value['marker']
#        self.style_dict['markersize'] = value['markersize']

class PlotsDataContainer:
    """
    Contains all plot data
    """
    def __init__(self):
        self.line_data_list = []
        self.patch_data_list = []
